@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include "compose.h"
 #include "manage_window.h"
 #include "stock_pixmap.h"
+#include "statusbar.h"
 #include "inc.h"
 #include "gtkutils.h"
 #include "utils.h"
@@ -385,8 +386,13 @@ void account_add(void)
 		folder->account = ac_prefs;
 		ac_prefs->folder = REMOTE_FOLDER(folder);
 		folder_add(folder);
-		if (ac_prefs->protocol == A_IMAP4)
-			folder->klass->create_tree(folder);
+		if (ac_prefs->protocol == A_IMAP4) {
+			if (main_window_toggle_online_if_offline
+				(main_window_get())) {
+				folder->klass->create_tree(folder);
+				statusbar_pop_all();
+			}
+		}
 		folderview_set_all();
 	}
 }
@@ -468,8 +474,13 @@ void account_set_missing_folder(void)
 			folder->account = ap;
 			ap->folder = REMOTE_FOLDER(folder);
 			folder_add(folder);
-			if (ap->protocol == A_IMAP4)
-				folder->klass->create_tree(folder);
+			if (ap->protocol == A_IMAP4) {
+				if (main_window_toggle_online_if_offline
+					(main_window_get())) {
+					folder->klass->create_tree(folder);
+					statusbar_pop_all();
+				}
+			}
 		}
 	}
 }
