@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@
 #define FOLDER_LIST		"folderlist.xml"
 #define CACHE_FILE		".sylpheed_cache"
 #define MARK_FILE		".sylpheed_mark"
-#define CACHE_VERSION		17
+#define CACHE_VERSION		18
 #define MARK_VERSION		2
 
 #define DEFAULT_SIGNATURE	".signature"
@@ -94,7 +94,9 @@
 #define MAX_ENTRY_LENGTH		8191
 #define COLOR_DIM			35000
 #define UI_REFRESH_INTERVAL		50000	/* usec */
-#define SESSION_TIMEOUT			60	/* sec */
+#define FOLDER_UPDATE_INTERVAL		1500	/* msec */
+#define PROGRESS_UPDATE_INTERVAL	200	/* msec */
+#define SESSION_TIMEOUT_INTERVAL	60	/* sec */
 #define MAX_HISTORY_SIZE		16
 
 #define DEFAULT_NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*-*-*-*-*-*-*," \
