@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@
 #include "alertpanel.h"
 #include "menu.h"
 #include "stock_pixmap.h"
+#include "statusbar.h"
 #include "procmsg.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -98,6 +99,8 @@ static GdkPixmap *folderxpm;
 static GdkBitmap *folderxpmmask;
 static GdkPixmap *folderopenxpm;
 static GdkBitmap *folderopenxpmmask;
+static GdkPixmap *foldernoselectxpm;
+static GdkBitmap *foldernoselectxpmmask;
 static GdkPixmap *trashxpm;
 static GdkBitmap *trashxpmmask;
 
@@ -112,15 +115,9 @@ static void folderview_append_folder	 (FolderView	*folderview,
 static void folderview_update_node	 (FolderView	*folderview,
 					  GtkCTreeNode	*node);
 
-static GtkCTreeNode *folderview_find_by_name	(GtkCTree	*ctree,
-						 GtkCTreeNode	*node,
-						 const gchar	*name);
-
 static gint folderview_clist_compare	(GtkCList	*clist,
 					 gconstpointer	 ptr1,
 					 gconstpointer	 ptr2);
-static gint folderview_compare_name	(gconstpointer	 a,
-					 gconstpointer	 b);
 
 /* callback functions */
 static void folderview_button_pressed	(GtkWidget	*ctree,
@@ -149,6 +146,10 @@ static void folderview_col_resized	(GtkCList	*clist,
 					 gint		 width,
 					 FolderView	*folderview);
 
+static void folderview_download_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void folderview_update_tree_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -166,9 +167,6 @@ static void folderview_remove_mailbox_cb(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
-static void folderview_new_imap_folder_cb(FolderView	*folderview,
-					  guint		 action,
-					  GtkWidget	*widget);
 static void folderview_rm_imap_server_cb (FolderView	*folderview,
 					  guint		 action,
 					  GtkWidget	*widget);
@@ -202,7 +200,7 @@ static void folderview_drag_leave_cb     (GtkWidget        *widget,
 					  guint             time,
 					  FolderView       *folderview);
 static void folderview_drag_received_cb  (GtkWidget        *widget,
-					  GdkDragContext   *drag_context,
+					  GdkDragContext   *context,
 					  gint              x,
 					  gint              y,
 					  GtkSelectionData *data,
@@ -228,9 +226,11 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 
 static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 {
-	{N_("/Create _new folder..."),	NULL, folderview_new_imap_folder_cb, 0, NULL},
-	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb,   0, NULL},
-	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb,   0, NULL},
+	{N_("/Create _new folder..."),	NULL, folderview_new_folder_cb,    0, NULL},
+	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
+	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
@@ -248,6 +248,8 @@ static GtkItemFactoryEntry folderview_news_popup_entries[] =
 					NULL, folderview_new_news_group_cb, 0, NULL},
 	{N_("/_Remove newsgroup"),	NULL, folderview_rm_news_group_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Down_load"),		NULL, folderview_download_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Check for new messages"),
 					NULL, folderview_update_tree_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
@@ -375,7 +377,7 @@ FolderView *folderview_create(void)
 	gtk_drag_dest_set(ctree, GTK_DEST_DEFAULT_ALL &
 			  ~GTK_DEST_DEFAULT_HIGHLIGHT,
 			  summary_drag_types, 1,
-			  GDK_ACTION_MOVE);
+			  GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(ctree), "drag_motion",
 			   GTK_SIGNAL_FUNC(folderview_drag_motion_cb),
 			   folderview);
@@ -415,6 +417,8 @@ void folderview_init(FolderView *folderview)
 			 &folderxpm, &folderxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_DIR_OPEN,
 			 &folderopenxpm, &folderopenxpmmask);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_DIR_NOSELECT,
+			 &foldernoselectxpm, &foldernoselectxpmmask);
 	stock_pixmap_gdk(ctree, STOCK_PIXMAP_TRASH, &trashxpm, &trashxpmmask);
 
 	if (!bold_style) {
@@ -568,6 +572,50 @@ void folderview_update_msg_num(FolderView *folderview, GtkCTreeNode *row)
 	folderview_update_node(folderview, row);
 }
 
+void folderview_append_item(FolderItem *item)
+{
+	FolderItem *parent;
+	GList *list;
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	g_return_if_fail(item->parent != NULL);
+
+	parent = item->parent;
+
+	for (list = folderview_list; list != NULL; list = list->next) {
+		FolderView *folderview = (FolderView *)list->data;
+		GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+		GtkCTreeNode *node, *child;
+
+		node = gtk_ctree_find_by_row_data(ctree, NULL, parent);
+		if (node) {
+			child = gtk_ctree_find_by_row_data(ctree, node, item);
+			if (!child) {
+				gchar *text[N_FOLDER_COLS] =
+					{NULL, "0", "0", "0"};
+
+				gtk_clist_freeze(GTK_CLIST(ctree));
+
+				text[COL_FOLDER] = item->name;
+				child = gtk_ctree_insert_node
+					(ctree, node, NULL, text,
+					 FOLDER_SPACING,
+					 folderxpm, folderxpmmask,
+					 folderopenxpm, folderopenxpmmask,
+					 FALSE, FALSE);
+				gtk_ctree_node_set_row_data(ctree, child, item);
+				gtk_ctree_expand(ctree, node);
+				folderview_update_node(folderview, child);
+				folderview_sort_folders(folderview, node,
+							item->folder);
+
+				gtk_clist_thaw(GTK_CLIST(ctree));
+			}
+		}
+	}
+}
+
 static void folderview_set_folders(FolderView *folderview)
 {
 	GList *list;
@@ -652,6 +700,10 @@ void folderview_rescan_tree(Folder *folder)
 		 _("Yes"), _("No"), NULL);
 	if (avalue != G_ALERTDEFAULT) return;
 
+	if (!FOLDER_IS_LOCAL(folder) &&
+	    !main_window_toggle_online_if_offline(main_window_get()))
+		return;
+
 	inc_lock();
 	window = label_window_create(_("Rebuilding folder tree..."));
 
@@ -662,6 +714,7 @@ void folderview_rescan_tree(Folder *folder)
 
 	folder_write_list();
 	folderview_set_all();
+	statusbar_pop_all();
 
 	gtk_widget_destroy(window);
 	inc_unlock();
@@ -705,6 +758,12 @@ void folderview_check_new(Folder *folder)
 		folderview = (FolderView *)list->data;
 		ctree = GTK_CTREE(folderview->ctree);
 
+		if (folder && !FOLDER_IS_LOCAL(folder)) {
+			if (!main_window_toggle_online_if_offline
+				(folderview->mainwin))
+				return;
+		}
+
 		inc_lock();
 		main_window_lock(folderview->mainwin);
 		gtk_widget_set_sensitive(folderview->ctree, FALSE);
@@ -728,6 +787,7 @@ void folderview_check_new(Folder *folder)
 		gtk_widget_set_sensitive(folderview->ctree, TRUE);
 		main_window_unlock(folderview->mainwin);
 		inc_unlock();
+		statusbar_pop_all();
 	}
 
 	folder_write_list();
@@ -904,10 +964,16 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 				item->name);
 		break;
 	default:
-		xpm = folderxpm;
-		mask = folderxpmmask;
-		openxpm = folderopenxpm;
-		openmask = folderopenxpmmask;
+		if (item->no_select) {
+			xpm = openxpm = foldernoselectxpm;
+			mask = openmask = foldernoselectxpmmask;
+		} else {
+			xpm = folderxpm;
+			mask = folderxpmmask;
+			openxpm = folderopenxpm;
+			openmask = folderopenxpmmask;
+		}
+
 		if (!item->parent) {
 			switch (FOLDER_TYPE(item->folder)) {
 			case F_MH:
@@ -992,7 +1058,10 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 
 	gtk_ctree_node_set_foreground(ctree, node, NULL);
 
-	if (use_bold && use_color)
+	if (item->no_select)
+		gtk_ctree_node_set_foreground(ctree, node,
+					      &folderview->color_noselect);
+	else if (use_bold && use_color)
 		style = bold_color_style;
 	else if (use_bold)
 		style = bold_style;
@@ -1002,6 +1071,8 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 
 	gtk_ctree_node_set_row_style(ctree, node, style);
 
+	item->updated = FALSE;
+
 	if ((node = gtkut_ctree_find_collapsed_parent(ctree, node)) != NULL)
 		folderview_update_node(folderview, node);
 }
@@ -1032,13 +1103,39 @@ void folderview_update_item(FolderItem *item, gboolean update_summary)
 static void folderview_update_item_foreach_func(gpointer key, gpointer val,
 						gpointer data)
 {
-	folderview_update_item((FolderItem *)key, (gboolean)data);
+	folderview_update_item((FolderItem *)key, GPOINTER_TO_INT(data));
 }
 
 void folderview_update_item_foreach(GHashTable *table, gboolean update_summary)
 {
 	g_hash_table_foreach(table, folderview_update_item_foreach_func,
-			     (gpointer)update_summary);
+			     GINT_TO_POINTER(update_summary));
+}
+
+static gboolean folderview_update_all_updated_func(GNode *node, gpointer data)
+{
+	FolderItem *item;
+
+	item = FOLDER_ITEM(node->data);
+	if (item->updated) {
+		debug_print("folderview_update_all_updated(): '%s' is updated\n", item->path);
+		folderview_update_item(item, GPOINTER_TO_INT(data));
+	}
+
+	return FALSE;
+}
+
+void folderview_update_all_updated(gboolean update_summary)
+{
+	GList *list;
+	Folder *folder;
+
+	for (list = folder_get_list(); list != NULL; list = list->next) {
+		folder = (Folder *)list->data;
+		g_node_traverse(folder->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+				folderview_update_all_updated_func,
+				GINT_TO_POINTER(update_summary));
+	}
 }
 
 static gboolean folderview_gnode_func(GtkCTree *ctree, guint depth,
@@ -1147,19 +1244,9 @@ void folderview_new_folder(FolderView *folderview)
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	switch (FOLDER_TYPE(item->folder)) {
-	case F_MH:
-	case F_MBOX:
-	case F_MAILDIR:
+	if (FOLDER_IS_LOCAL(item->folder) ||
+	    FOLDER_TYPE(item->folder) == F_IMAP)
 		folderview_new_folder_cb(folderview, 0, NULL);
-		break;
-	case F_IMAP:
-		folderview_new_imap_folder_cb(folderview, 0, NULL);
-		break;
-	case F_NEWS:
-	default:
-		break;
-	}
 }
 
 void folderview_rename_folder(FolderView *folderview)
@@ -1228,6 +1315,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	gboolean new_folder      = FALSE;
 	gboolean rename_folder   = FALSE;
 	gboolean delete_folder   = FALSE;
+	gboolean download_msg    = FALSE;
 	gboolean update_tree     = FALSE;
 	gboolean rescan_tree     = FALSE;
 	gboolean remove_tree     = FALSE;
@@ -1280,8 +1368,11 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 			update_tree = remove_tree = TRUE;
 			if (folder->account)
 				folder_property = TRUE;
-		} else
-			search_folder = folder_property = TRUE;
+		} else {
+			folder_property = TRUE;
+			if (folderview->selected == folderview->opened)
+				search_folder = TRUE;
+		}
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
 				update_tree = rescan_tree = TRUE;
@@ -1291,6 +1382,11 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 			if (item->parent != NULL)
 				delete_folder = TRUE;
 		}
+		if (FOLDER_TYPE(folder) == F_IMAP ||
+		    FOLDER_TYPE(folder) == F_NEWS) {
+			if (item->parent != NULL && item->no_select == FALSE)
+				download_msg = TRUE;
+		}
 	}
 
 #define SET_SENS(factory, name, sens) \
@@ -1313,6 +1409,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(imap_factory, "/Create new folder...", new_folder);
 		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
 		SET_SENS(imap_factory, "/Delete folder", delete_folder);
+		SET_SENS(imap_factory, "/Download", download_msg);
 		SET_SENS(imap_factory, "/Check for new messages", update_tree);
 		SET_SENS(imap_factory, "/Rebuild folder tree", rescan_tree);
 		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
@@ -1323,6 +1420,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
 		SET_SENS(news_factory, "/Subscribe to newsgroup...", new_folder);
 		SET_SENS(news_factory, "/Remove newsgroup", delete_folder);
+		SET_SENS(news_factory, "/Download", download_msg);
 		SET_SENS(news_factory, "/Check for new messages", update_tree);
 		SET_SENS(news_factory, "/Remove news account", remove_tree);
 		SET_SENS(news_factory, "/Search messages...", search_folder);
@@ -1489,27 +1587,63 @@ static void folderview_col_resized(GtkCList *clist, gint column, gint width,
 	}
 }
 
-static GtkCTreeNode *folderview_find_by_name(GtkCTree *ctree,
-					     GtkCTreeNode *node,
-					     const gchar *name)
+static void folderview_download_func(Folder *folder, FolderItem *item,
+				     gpointer data)
 {
+	GList *list;
+
+	for (list = folderview_list; list != NULL; list = list->next) {
+		FolderView *folderview = (FolderView *)list->data;
+		MainWindow *mainwin = folderview->mainwin;
+		gchar *str;
+
+		str = g_strdup_printf
+			(_("Downloading messages in %s ..."), item->path);
+		main_window_progress_set(mainwin,
+					 GPOINTER_TO_INT(data), item->total);
+		STATUSBAR_PUSH(mainwin, str);
+		STATUSBAR_POP(mainwin);
+		g_free(str);
+	}
+}
+
+static void folderview_download_cb(FolderView *folderview, guint action,
+				   GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	MainWindow *mainwin = folderview->mainwin;
 	FolderItem *item;
 
-	if (!node)
-		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
-	if (!node)
-		return NULL;
+	if (!folderview->selected) return;
 
-	node = GTK_CTREE_ROW(node)->children;
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
 
-	while (node) {
-		item = gtk_ctree_node_get_row_data(ctree, node);
-		if (!folderview_compare_name(item, name))
-			return node;
-		node = GTK_CTREE_ROW(node)->sibling;
+	if (!main_window_toggle_online_if_offline(folderview->mainwin))
+		return;
+
+	main_window_cursor_wait(mainwin);
+	inc_lock();
+	main_window_lock(mainwin);
+	gtk_widget_set_sensitive(folderview->ctree, FALSE);
+	main_window_progress_on(mainwin);
+	GTK_EVENTS_FLUSH();
+	folder_set_ui_func(item->folder, folderview_download_func, NULL);
+	if (folder_item_fetch_all_msg(item) < 0) {
+		gchar *name;
+
+		name = trim_string(item->name, 32);
+		alertpanel_error(_("Error occurred while downloading messages in `%s'."), name);
+		g_free(name);
 	}
-
-	return NULL;
+	folder_set_ui_func(item->folder, NULL, NULL);
+	main_window_progress_off(mainwin);
+	gtk_widget_set_sensitive(folderview->ctree, TRUE);
+	main_window_unlock(mainwin);
+	inc_unlock();
+	main_window_cursor_normal(mainwin);
+	statusbar_pop_all();
 }
 
 static void folderview_update_tree_cb(FolderView *folderview, guint action,
@@ -1536,39 +1670,49 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	FolderItem *item;
 	FolderItem *new_item;
 	gchar *new_folder;
-	gchar *name, *name_;
-	GtkCTreeNode *node;
+	gchar *name;
+	gchar *p;
 
 	if (!folderview->selected) return;
 
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
+	if (FOLDER_TYPE(item->folder) == F_IMAP)
+		g_return_if_fail(item->folder->account != NULL);
 
-	new_folder = input_dialog(_("New folder"),
-				  _("Input the name of new folder:"),
-				  _("NewFolder"));
+	if (FOLDER_TYPE(item->folder) == F_IMAP) {
+		new_folder = input_dialog
+			(_("New folder"),
+			 _("Input the name of new folder:\n"
+			   "(if you want to create a folder to store subfolders,\n"
+			   " append `/' at the end of the name)"),
+			 _("NewFolder"));
+	} else {
+		new_folder = input_dialog(_("New folder"),
+					  _("Input the name of new folder:"),
+					  _("NewFolder"));
+	}
 	if (!new_folder) return;
+	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
-	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
+	p = strchr(new_folder, G_DIR_SEPARATOR);
+	if ((p && FOLDER_TYPE(item->folder) != F_IMAP) ||
+	    (p && FOLDER_TYPE(item->folder) == F_IMAP && *(p + 1) != '\0')) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
-		g_free(new_folder);
 		return;
 	}
 
-	name_ = trim_string(new_folder, 32);
-	Xstrdup_a(name, name_, {g_free(new_folder); return;});
-	g_free(name_);
+	name = trim_string(new_folder, 32);
+	AUTORELEASE_STR(name, {g_free(name); return;});
 
 	/* find whether the directory already exists */
-	if (folderview_find_by_name(ctree, folderview->selected, new_folder)) {
+	if (folder_find_child_item_by_name(item, new_folder)) {
 		alertpanel_error(_("The folder `%s' already exists."), name);
-		g_free(new_folder);
 		return;
 	}
 
@@ -1576,25 +1720,10 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 						      new_folder);
 	if (!new_item) {
 		alertpanel_error(_("Can't create the folder `%s'."), name);
-		g_free(new_folder);
 		return;
 	}
-	g_free(new_folder);
-
-	gtk_clist_freeze(GTK_CLIST(ctree));
-
-	text[COL_FOLDER] = new_item->name;
-	node = gtk_ctree_insert_node(ctree, folderview->selected, NULL, text,
-				     FOLDER_SPACING,
-				     folderxpm, folderxpmmask,
-				     folderopenxpm, folderopenxpmmask,
-				     FALSE, FALSE);
-	gtk_ctree_expand(ctree, folderview->selected);
-	gtk_ctree_node_set_row_data(ctree, node, new_item);
-	folderview_sort_folders(folderview, folderview->selected, item->folder);
-
-	gtk_clist_thaw(GTK_CLIST(ctree));
 
+	folderview_append_item(new_item);
 	folder_write_list();
 }
 
@@ -1604,7 +1733,7 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	gchar *new_folder;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	gchar *old_path;
 	gchar *old_id;
@@ -1617,29 +1746,25 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->path != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	name_ = trim_string(item->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->name, 32);
 	message = g_strdup_printf(_("Input new name for `%s':"), name);
 	new_folder = input_dialog(_("Rename folder"), message,
 				  g_basename(item->path));
 	g_free(message);
+	g_free(name);
 	if (!new_folder) return;
+	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
 	if (strchr(new_folder, G_DIR_SEPARATOR) != NULL) {
 		alertpanel_error(_("`%c' can't be included in folder name."),
 				 G_DIR_SEPARATOR);
-		g_free(new_folder);
 		return;
 	}
 
-	if (folderview_find_by_name
-		(ctree, GTK_CTREE_ROW(folderview->selected)->parent,
-		 new_folder)) {
+	if (folder_find_child_item_by_name(item->parent, new_folder)) {
 		name = trim_string(new_folder, 32);
 		alertpanel_error(_("The folder `%s' already exists."), name);
 		g_free(name);
-		g_free(new_folder);
 		return;
 	}
 
@@ -1649,10 +1774,8 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 	if (item->folder->klass->rename_folder(item->folder, item,
 					       new_folder) < 0) {
 		g_free(old_id);
-		g_free(new_folder);
 		return;
 	}
-	g_free(new_folder);
 
 	if (folder_get_default_folder() == item->folder)
 		prefs_filter_rename_path(old_path, item->path);
@@ -1687,7 +1810,7 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	Folder *folder;
 	FolderItem *item;
-	gchar *message, *name, *name_;
+	gchar *message, *name;
 	AlertValue avalue;
 	gchar *old_path;
 	gchar *old_id;
@@ -1701,9 +1824,8 @@ static void folderview_delete_folder_cb(FolderView *folderview, guint action,
 
 	folder = item->folder;
 
-	name_ = trim_string(item->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->name, 32);
+	AUTORELEASE_STR(name, {g_free(name); return;});
 	message = g_strdup_printf
 		(_("All folder(s) and message(s) under `%s' will be deleted.\n"
 		   "Do you really want to delete?"), name);
@@ -1743,7 +1865,7 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	GtkCTreeNode *node;
 	FolderItem *item;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -1754,15 +1876,14 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item->folder != NULL);
 	if (item->parent) return;
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf
 		(_("Really remove the mailbox `%s' ?\n"
 		   "(The messages are NOT deleted from the disk)"), name);
 	avalue = alertpanel(_("Remove mailbox"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (folderview->summaryview->folder_item &&
@@ -1775,86 +1896,13 @@ static void folderview_remove_mailbox_cb(FolderView *folderview, guint action,
 	folder_write_list();
 }
 
-static void folderview_new_imap_folder_cb(FolderView *folderview, guint action,
-					  GtkWidget *widget)
-{
-	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
-	GtkCTreeNode *node;
-	FolderItem *item;
-	FolderItem *new_item;
-	gchar *new_folder;
-	gchar *name, *name_;
-	gchar *p;
-
-	if (!folderview->selected) return;
-
-	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
-	g_return_if_fail(item->folder->account != NULL);
-
-	new_folder = input_dialog
-		(_("New folder"),
-		 _("Input the name of new folder:\n"
-		   "(if you want to create a folder to store subfolders,\n"
-		   " append `/' at the end of the name)"),
-		 _("NewFolder"));
-	if (!new_folder) return;
-
-	if ((p = strchr(new_folder, G_DIR_SEPARATOR)) != NULL &&
-	    *(p + 1) != '\0') {
-		alertpanel_error(_("`%c' can't be included in folder name."),
-				 G_DIR_SEPARATOR);
-		g_free(new_folder);
-		return;
-	}
-
-	name_ = trim_string(new_folder, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
-
-	/* find whether the directory already exists */
-	if (folderview_find_by_name(ctree, folderview->selected, new_folder)) {
-		alertpanel_error(_("The folder `%s' already exists."), name);
-		g_free(new_folder);
-		return;
-	}
-
-	new_item = item->folder->klass->create_folder(item->folder, item,
-						      new_folder);
-	if (!new_item) {
-		alertpanel_error(_("Can't create the folder `%s'."), name);
-		g_free(new_folder);
-		return;
-	}
-	g_free(new_folder);
-
-	gtk_clist_freeze(GTK_CLIST(ctree));
-
-	text[COL_FOLDER] = new_item->name;
-	node = gtk_ctree_insert_node(ctree, folderview->selected, NULL, text,
-				     FOLDER_SPACING,
-				     folderxpm, folderxpmmask,
-				     folderopenxpm, folderopenxpmmask,
-				     FALSE, FALSE);
-	gtk_ctree_expand(ctree, folderview->selected);
-	gtk_ctree_node_set_row_data(ctree, node, new_item);
-	folderview_sort_folders(folderview, folderview->selected, item->folder);
-
-	gtk_clist_thaw(GTK_CLIST(ctree));
-
-	folder_write_list();
-}
-
 static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	PrefsAccount *account;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -1866,13 +1914,12 @@ static void folderview_rm_imap_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_IMAP);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete IMAP4 account `%s'?"), name);
 	avalue = alertpanel(_("Delete IMAP4 account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 
 	if (avalue != G_ALERTDEFAULT) return;
 
@@ -1895,7 +1942,6 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 					 GtkWidget *widget)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
-	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	GtkCTreeNode *servernode, *node;
 	Folder *folder;
 	FolderItem *item;
@@ -1957,23 +2003,14 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 	for (cur = new_subscr; cur != NULL; cur = cur->next) {
 		gchar *name = (gchar *)cur->data;
 
-		if (folderview_find_by_name(ctree, servernode, name) != NULL)
+		if (folder_find_child_item_by_name(rootitem, name) != NULL)
 			continue;
 
-		text[COL_FOLDER] = name;
-		node = gtk_ctree_insert_node(ctree, servernode, NULL, text,
-					     FOLDER_SPACING,
-					     folderxpm, folderxpmmask,
-					     folderopenxpm, folderopenxpmmask,
-					     FALSE, FALSE);
-		gtk_ctree_expand(ctree, servernode);
-
 		newitem = folder_item_new(name, name);
 		folder_item_append(rootitem, newitem);
-		gtk_ctree_node_set_row_data(ctree, node, newitem);
+		folderview_append_item(newitem);
 	}
 
-	folderview_sort_folders(folderview, servernode, folder);
 	gtk_clist_thaw(GTK_CLIST(ctree));
 
 	slist_free_strings(new_subscr);
@@ -1987,7 +2024,7 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -1999,13 +2036,12 @@ static void folderview_rm_news_group_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->path, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->path, 32);
 	message = g_strdup_printf(_("Really delete newsgroup `%s'?"), name);
 	avalue = alertpanel(_("Delete newsgroup"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 	if (avalue != G_ALERTDEFAULT) return;
 
 	if (folderview->opened == folderview->selected) {
@@ -2024,7 +2060,7 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	FolderItem *item;
 	PrefsAccount *account;
-	gchar *name, *name_;
+	gchar *name;
 	gchar *message;
 	AlertValue avalue;
 
@@ -2036,13 +2072,12 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	g_return_if_fail(FOLDER_TYPE(item->folder) == F_NEWS);
 	g_return_if_fail(item->folder->account != NULL);
 
-	name_ = trim_string(item->folder->name, 32);
-	Xstrdup_a(name, name_, return);
-	g_free(name_);
+	name = trim_string(item->folder->name, 32);
 	message = g_strdup_printf(_("Really delete news account `%s'?"), name);
 	avalue = alertpanel(_("Delete news account"), message,
 			    _("Yes"), _("+No"), NULL);
 	g_free(message);
+	g_free(name);
 
 	if (avalue != G_ALERTDEFAULT) return;
 
@@ -2085,6 +2120,28 @@ static void folderview_property_cb(FolderView *folderview, guint action,
 		prefs_folder_item_open(item);
 }
 
+static void folderview_defer_expand_stop(FolderView *folderview)
+{
+	if (folderview->spring_timer > 0) {
+		gtk_timeout_remove(folderview->spring_timer);
+		folderview->spring_timer = 0;
+	}
+	folderview->spring_node = NULL;
+}
+
+static gint folderview_defer_expand(gpointer data)
+{
+	FolderView *folderview = (FolderView *)data;
+
+	if (folderview->spring_node) {
+		gtk_ctree_expand(GTK_CTREE(folderview->ctree),
+				 folderview->spring_node);
+	}
+	folderview_defer_expand_stop(folderview);
+
+	return FALSE;
+}
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
@@ -2102,16 +2159,18 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 		item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 		src_item = folderview->summaryview->folder_item;
-		if (item && item->folder && item->path &&
-		    src_item && src_item != item) {
-			switch (FOLDER_TYPE(item->folder)) {
-			case F_MH:
-			case F_IMAP:
-				acceptable = TRUE;
-				break;
-			default:
-				break;
-			}
+		if (src_item && src_item != item)
+			acceptable = FOLDER_ITEM_CAN_ADD(item);
+	}
+
+	if (node != folderview->spring_node) {
+		folderview_defer_expand_stop(folderview);
+		if (node && !GTK_CTREE_ROW(node)->expanded &&
+		    GTK_CTREE_ROW(node)->children) {
+			folderview->spring_timer =
+				gtk_timeout_add(1000, folderview_defer_expand,
+						folderview);
+			folderview->spring_node = node;
 		}
 	}
 
@@ -2123,7 +2182,14 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		gtk_signal_handler_unblock_by_func
 			(GTK_OBJECT(widget),
 			 GTK_SIGNAL_FUNC(folderview_selected), folderview);
-		gdk_drag_status(context, context->suggested_action, time);
+		if ((context->actions & GDK_ACTION_MOVE) != 0)
+			gdk_drag_status(context, GDK_ACTION_MOVE, time);
+		else if ((context->actions & GDK_ACTION_COPY) != 0)
+			gdk_drag_status(context, GDK_ACTION_COPY, time);
+		else if ((context->actions & GDK_ACTION_LINK) != 0)
+			gdk_drag_status(context, GDK_ACTION_LINK, time);
+		else
+			gdk_drag_status(context, 0, time);
 	} else {
 		gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 		gdk_drag_status(context, 0, time);
@@ -2137,11 +2203,12 @@ static void folderview_drag_leave_cb(GtkWidget      *widget,
 				     guint           time,
 				     FolderView     *folderview)
 {
+	folderview_defer_expand_stop(folderview);
 	gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
 }
 
 static void folderview_drag_received_cb(GtkWidget        *widget,
-					GdkDragContext   *drag_context,
+					GdkDragContext   *context,
 					gint              x,
 					gint              y,
 					GtkSelectionData *data,
@@ -2153,6 +2220,8 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	FolderItem *item, *src_item;
 	GtkCTreeNode *node;
 
+	folderview_defer_expand_stop(folderview);
+
 	if (gtk_clist_get_selection_info
 		(GTK_CLIST(widget), x - 24, y - 24, &row, &column) == 0)
 		return;
@@ -2160,17 +2229,17 @@ static void folderview_drag_received_cb(GtkWidget        *widget,
 	node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 	item = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 	src_item = folderview->summaryview->folder_item;
-	if (item && src_item) {
-		if (FOLDER_TYPE(src_item->folder) !=
-		    FOLDER_TYPE(item->folder) ||
-		    (FOLDER_TYPE(item->folder) == F_IMAP &&
-		     src_item->folder != item->folder))
-			summary_copy_selected_to(folderview->summaryview, item);
-		else
+	if (FOLDER_ITEM_CAN_ADD(item) && src_item && src_item != item) {
+		if ((context->actions & GDK_ACTION_MOVE) != 0) {
 			summary_move_selected_to(folderview->summaryview, item);
-		gtk_drag_finish(drag_context, TRUE, TRUE, time);
+			gtk_drag_finish(context, TRUE, TRUE, time);
+		} else if ((context->actions & GDK_ACTION_COPY) != 0) {
+			summary_copy_selected_to(folderview->summaryview, item);
+			gtk_drag_finish(context, TRUE, TRUE, time);
+		} else
+			gtk_drag_finish(context, FALSE, FALSE, time);
 	} else
-		gtk_drag_finish(drag_context, FALSE, FALSE, time);
+		gtk_drag_finish(context, FALSE, FALSE, time);
 }
 
 static gint folderview_clist_compare(GtkCList *clist,
@@ -2186,12 +2255,3 @@ static gint folderview_clist_compare(GtkCList *clist,
 
 	return g_strcasecmp(item1->name, item2->name);
 }
-
-static gint folderview_compare_name(gconstpointer a, gconstpointer b)
-{
-	const FolderItem *item = a;
-	const gchar *name = b;
-
-	if (!item->path) return -1;
-	return strcmp2(g_basename(item->path), name);
-}
