@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,12 +43,12 @@
 #include "procmsg.h"
 #include "procheader.h"
 #include "folder.h"
-#include "statusbar.h"
 #include "prefs_account.h"
 #include "codeconv.h"
 #include "md5.h"
 #include "base64.h"
 #include "utils.h"
+#include "prefs_common.h"
 #include "inputdialog.h"
 
 #define IMAP4_PORT	143
@@ -87,6 +87,13 @@ static Session *imap_session_new	(PrefsAccount	*account);
 static void imap_session_destroy	(Session	*session);
 /* static void imap_session_destroy_all	(void); */
 
+static gint imap_search_flags		(IMAPSession	*session,
+					 GArray	       **uids,
+					 GHashTable    **flags_table);
+static gint imap_fetch_flags		(IMAPSession	*session,
+					 GArray	       **uids,
+					 GHashTable    **flags_table);
+
 static GSList *imap_get_msg_list	(Folder		*folder,
 					 FolderItem	*item,
 					 gboolean	 use_cache);
@@ -175,11 +182,15 @@ static gint imap_do_copy_msgs		(Folder		*folder,
 					 FolderItem	*dest, 
 					 GSList		*msglist,
 					 gboolean	 remove_source);
+static gint imap_remove_msgs_by_seq_set	(Folder		*folder,
+					 FolderItem	*item,
+					 GSList		*seq_list);
 
 static GSList *imap_get_uncached_messages	(IMAPSession	*session,
 						 FolderItem	*item,
 						 guint32	 first_uid,
-						 guint32	 last_uid);
+						 guint32	 last_uid,
+						 gboolean	 update_count);
 static void imap_delete_cached_message		(FolderItem	*item,
 						 guint32	 uid);
 static GSList *imap_delete_cached_messages	(GSList		*mlist,
@@ -238,6 +249,7 @@ static gchar *imap_parse_atom		(IMAPSession	*session,
 					 gint		 dest_len,
 					 GString	*str);
 static MsgFlags imap_parse_flags	(const gchar	*flag_str);
+static IMAPFlags imap_parse_imap_flags	(const gchar	*flag_str);
 static MsgInfo *imap_parse_envelope	(IMAPSession	*session,
 					 FolderItem	*item,
 					 GString	*line_str);
@@ -421,6 +433,9 @@ static IMAPSession *imap_session_get(Folder *folder)
 	g_return_val_if_fail(FOLDER_TYPE(folder) == F_IMAP, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
+	if (!prefs_common.online_mode)
+		return NULL;
+
 	if (!rfolder->session) {
 		rfolder->session = imap_session_new(folder->account);
 		if (rfolder->session) {
@@ -428,13 +443,11 @@ static IMAPSession *imap_session_get(Folder *folder)
 					     IMAP_FOLDER(folder));
 			rfolder->session->last_access_time = time(NULL);
 		}
-		statusbar_pop_all();
 		return IMAP_SESSION(rfolder->session);
 	}
 
-	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT) {
+	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT_INTERVAL) {
 		rfolder->session->last_access_time = time(NULL);
-		statusbar_pop_all();
 		return IMAP_SESSION(rfolder->session);
 	}
 
@@ -451,7 +464,6 @@ static IMAPSession *imap_session_get(Folder *folder)
 
 	if (rfolder->session)
 		rfolder->session->last_access_time = time(NULL);
-	statusbar_pop_all();
 	return IMAP_SESSION(rfolder->session);
 }
 
@@ -613,6 +625,157 @@ static void imap_session_destroy_all(void)
 
 #define THROW goto catch
 
+static gint imap_search_flags(IMAPSession *session, GArray **uids,
+			      GHashTable **flags_table)
+{
+	gint ok;
+	gint i;
+	GArray *flag_uids;
+	GHashTable *unseen_table;
+	GHashTable *flagged_table;
+	GHashTable *answered_table;
+	guint32 uid;
+	IMAPFlags flags;
+
+	ok = imap_cmd_search(session, "ALL", uids);
+	if (ok != IMAP_SUCCESS) return ok;
+
+	ok = imap_cmd_search(session, "UNSEEN", &flag_uids);
+	if (ok != IMAP_SUCCESS) {
+		g_array_free(*uids, TRUE);
+		return ok;
+	}
+	unseen_table = imap_get_uid_table(flag_uids);
+	g_array_free(flag_uids, TRUE);
+	ok = imap_cmd_search(session, "FLAGGED", &flag_uids);
+	if (ok != IMAP_SUCCESS) {
+		g_hash_table_destroy(unseen_table);
+		g_array_free(*uids, TRUE);
+		return ok;
+	}
+	flagged_table = imap_get_uid_table(flag_uids);
+	g_array_free(flag_uids, TRUE);
+	ok = imap_cmd_search(session, "ANSWERED", &flag_uids);
+	if (ok != IMAP_SUCCESS) {
+		g_hash_table_destroy(flagged_table);
+		g_hash_table_destroy(unseen_table);
+		g_array_free(*uids, TRUE);
+		return ok;
+	}
+	answered_table = imap_get_uid_table(flag_uids);
+	g_array_free(flag_uids, TRUE);
+
+	*flags_table = g_hash_table_new(NULL, g_direct_equal);
+
+	for (i = 0; i < (*uids)->len; i++) {
+		uid = g_array_index(*uids, guint32, i);
+		flags = IMAP_FLAG_DRAFT;
+		if (!g_hash_table_lookup(unseen_table, GUINT_TO_POINTER(uid)))
+			flags |= IMAP_FLAG_SEEN;
+		if (g_hash_table_lookup(flagged_table, GUINT_TO_POINTER(uid)))
+			flags |= IMAP_FLAG_FLAGGED;
+		if (g_hash_table_lookup(answered_table, GUINT_TO_POINTER(uid)))
+			flags |= IMAP_FLAG_ANSWERED;
+		g_hash_table_insert(*flags_table, GUINT_TO_POINTER(uid),
+				    GINT_TO_POINTER(flags));
+	}
+
+	g_hash_table_destroy(answered_table);
+	g_hash_table_destroy(flagged_table);
+	g_hash_table_destroy(unseen_table);
+
+	return IMAP_SUCCESS;
+}
+
+static gint imap_fetch_flags(IMAPSession *session, GArray **uids,
+			     GHashTable **flags_table)
+{
+	gint ok;
+	gchar *tmp;
+	gchar *cur_pos;
+	gchar buf[IMAPBUFSIZE];
+	guint32 uid;
+	IMAPFlags flags;
+
+	imap_cmd_gen_send(session, "UID FETCH 1:* (UID FLAGS)");
+
+	*uids = g_array_new(FALSE, FALSE, sizeof(guint32));
+	*flags_table = g_hash_table_new(NULL, g_direct_equal);
+
+	while ((ok = imap_cmd_gen_recv(session, &tmp)) == IMAP_SUCCESS) {
+		if (tmp[0] != '*' || tmp[1] != ' ') {
+			g_free(tmp);
+			break;
+		}
+		cur_pos = tmp + 2;
+
+#define PARSE_ONE_ELEMENT(ch)					\
+{								\
+	cur_pos = strchr_cpy(cur_pos, ch, buf, sizeof(buf));	\
+	if (cur_pos == NULL) {					\
+		g_warning("cur_pos == NULL\n");			\
+		g_free(tmp);					\
+		g_hash_table_destroy(*flags_table);		\
+		g_array_free(*uids, TRUE);			\
+		return IMAP_ERROR;				\
+	}							\
+}
+
+		PARSE_ONE_ELEMENT(' ');
+		PARSE_ONE_ELEMENT(' ');
+		if (strcmp(buf, "FETCH") != 0) {
+			g_free(tmp);
+			continue;
+		}
+		if (*cur_pos != '(') {
+			g_free(tmp);
+			continue;
+		}
+		cur_pos++;
+		uid = 0;
+		flags = 0;
+
+		while (*cur_pos != '\0' && *cur_pos != ')') {
+			while (*cur_pos == ' ') cur_pos++;
+
+			if (!strncmp(cur_pos, "UID ", 4)) {
+				cur_pos += 4;
+				uid = strtoul(cur_pos, &cur_pos, 10);
+			} else if (!strncmp(cur_pos, "FLAGS ", 6)) {
+				cur_pos += 6;
+				if (*cur_pos != '(') {
+					g_warning("*cur_pos != '('\n");
+					break;
+				}
+				cur_pos++;
+				PARSE_ONE_ELEMENT(')');
+				flags = imap_parse_imap_flags(buf);
+				flags |= IMAP_FLAG_DRAFT;
+			} else {
+				g_warning("invalid FETCH response: %s\n", cur_pos);
+				break;
+			}
+		}
+
+#undef PARSE_ONE_ELEMENT
+
+		if (uid > 0) {
+			g_array_append_val(*uids, uid);
+			g_hash_table_insert(*flags_table, GUINT_TO_POINTER(uid),
+					    GINT_TO_POINTER(flags));
+		}
+
+		g_free(tmp);
+	}
+
+	if (ok != IMAP_SUCCESS) {
+		g_hash_table_destroy(*flags_table);
+		g_array_free(*uids, TRUE);
+	}
+
+	return ok;
+}
+
 static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 				 gboolean use_cache)
 {
@@ -635,7 +798,6 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 		mlist = procmsg_read_cache(item, FALSE);
 		item->last_num = procmsg_get_last_num_in_msg_list(mlist);
 		procmsg_set_flags(mlist, item);
-		statusbar_pop_all();
 		return mlist;
 	}
 
@@ -645,7 +807,6 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 
 	if (exists == 0) {
 		imap_delete_all_cached_messages(item);
-		statusbar_pop_all();
 		return NULL;
 	}
 
@@ -659,39 +820,44 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 	if (use_cache) {
 		GArray *uids;
 		GHashTable *msg_table;
-		GHashTable *uid_table;
-		GHashTable *unseen_table = NULL, *flagged_table = NULL,
-			   *answered_table = NULL;
+		GHashTable *flags_table;
 		guint32 cache_last;
 		guint32 begin = 0;
 		GSList *cur, *next = NULL;
 		MsgInfo *msginfo;
+		IMAPFlags imap_flags;
 
 		/* get cache data */
 		mlist = procmsg_read_cache(item, FALSE);
 		procmsg_set_flags(mlist, item);
 		cache_last = procmsg_get_last_num_in_msg_list(mlist);
 
-		/* get all UID list */
-		ok = imap_cmd_search(session, "ALL", &uids);
-		if (ok != IMAP_SUCCESS) THROW;
+		/* get all UID list and flags */
+		ok = imap_search_flags(session, &uids, &flags_table);
+		if (ok != IMAP_SUCCESS) {
+			if (ok == IMAP_SOCKET || ok == IMAP_IOERR) THROW;
+			ok = imap_fetch_flags(session, &uids, &flags_table);
+			if (ok != IMAP_SUCCESS) THROW;
+		}
 
 		if (uids->len > 0) {
 			first_uid = g_array_index(uids, guint32, 0);
 			last_uid = g_array_index(uids, guint32, uids->len - 1);
 		} else {
 			g_array_free(uids, TRUE);
+			g_hash_table_destroy(flags_table);
 			THROW;
 		}
 
-		/* check for deleted messages */
-		uid_table = imap_get_uid_table(uids);
+		/* sync message flags with server */
 		for (cur = mlist; cur != NULL; cur = next) {
 			msginfo = (MsgInfo *)cur->data;
 			next = cur->next;
-			if (g_hash_table_lookup
-				(uid_table, GUINT_TO_POINTER(msginfo->msgnum))
-				== NULL) {
+			imap_flags = GPOINTER_TO_INT(g_hash_table_lookup
+				(flags_table,
+				 GUINT_TO_POINTER(msginfo->msgnum)));
+
+			if (imap_flags == 0) {
 				debug_print("imap_get_msg_list: "
 					    "message %u has been deleted.\n",
 					    msginfo->msgnum);
@@ -704,66 +870,37 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 				item->total--;
 				mlist = g_slist_remove(mlist, msginfo);
 				procmsg_msginfo_free(msginfo);
+				continue;
+			}
+
+			if (!IMAP_IS_SEEN(imap_flags)) {
+				if (!MSG_IS_UNREAD(msginfo->flags)) {
+					item->unread++;
+					MSG_SET_PERM_FLAGS(msginfo->flags,
+							   MSG_UNREAD);
+				}
+			} else {
+				if (MSG_IS_NEW(msginfo->flags))
+					item->new--;
+				if (MSG_IS_UNREAD(msginfo->flags))
+					item->unread--;
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_NEW|MSG_UNREAD);
+			}
+
+			if (IMAP_IS_FLAGGED(imap_flags)) {
+				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_MARKED);
+			} else {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_MARKED);
+			}
+			if (IMAP_IS_ANSWERED(imap_flags)) {
+				MSG_SET_PERM_FLAGS(msginfo->flags, MSG_REPLIED);
+			} else {
+				MSG_UNSET_PERM_FLAGS(msginfo->flags,
+						     MSG_REPLIED);
 			}
 		}
-		g_hash_table_destroy(uid_table);
-
-#define GET_FLAG_UID_TABLE(flag, table)					  \
-{									  \
-	GArray *flag_uids;						  \
-									  \
-	ok = imap_cmd_search(session, flag, &flag_uids);		  \
-	if (ok != IMAP_SUCCESS) {					  \
-		if (unseen_table) g_hash_table_destroy(unseen_table);	  \
-		if (flagged_table) g_hash_table_destroy(flagged_table);	  \
-		if (answered_table) g_hash_table_destroy(answered_table); \
-		g_array_free(uids, TRUE);				  \
-		THROW;							  \
-	}								  \
-	table = imap_get_uid_table(flag_uids);				  \
-	g_array_free(flag_uids, TRUE);					  \
-}
-
-#define LOOKUP_AND_SET_FLAGS(table, set_flags, unset_flags)		\
-{									\
-	if (g_hash_table_lookup						\
-		(table, GUINT_TO_POINTER(msginfo->msgnum)) != NULL) {	\
-		if (set_flags == MSG_UNREAD &&				\
-		    !MSG_IS_UNREAD(msginfo->flags))			\
-			item->unread++;					\
-		MSG_SET_PERM_FLAGS(msginfo->flags, set_flags);		\
-	} else {							\
-		if ((unset_flags) == (MSG_NEW|MSG_UNREAD)) {		\
-			if (MSG_IS_NEW(msginfo->flags))			\
-				item->new--;				\
-			if (MSG_IS_UNREAD(msginfo->flags))		\
-				item->unread--;				\
-		}							\
-		MSG_UNSET_PERM_FLAGS(msginfo->flags, unset_flags);	\
-	}								\
-}
-
-		/* sync flags with server */
-		GET_FLAG_UID_TABLE("UNSEEN", unseen_table);
-		GET_FLAG_UID_TABLE("FLAGGED", flagged_table);
-		GET_FLAG_UID_TABLE("ANSWERED", answered_table);
-
-		for (cur = mlist; cur != NULL; cur = cur->next) {
-			msginfo = (MsgInfo *)cur->data;
-			LOOKUP_AND_SET_FLAGS
-				(unseen_table, MSG_UNREAD, MSG_NEW|MSG_UNREAD);
-			LOOKUP_AND_SET_FLAGS
-				(flagged_table, MSG_MARKED, MSG_MARKED);
-			LOOKUP_AND_SET_FLAGS
-				(answered_table, MSG_REPLIED, MSG_REPLIED);
-		}
-
-#undef GET_FLAG_UID_TABLE
-#undef LOOKUP_AND_SET_FLAGS
-
-		g_hash_table_destroy(unseen_table);
-		g_hash_table_destroy(flagged_table);
-		g_hash_table_destroy(answered_table);
 
 		/* check for the first new message */
 		msg_table = procmsg_msg_hash_table_create(mlist);
@@ -789,6 +926,7 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 		}
 
 		g_array_free(uids, TRUE);
+		g_hash_table_destroy(flags_table);
 
 		/* remove ununsed caches */
 		if (first_uid > 0 && last_uid > 0) {
@@ -802,12 +940,13 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 		if (begin > 0 && begin <= last_uid) {
 			GSList *newlist;
 			newlist = imap_get_uncached_messages(session, item,
-							     begin, last_uid);
+							     begin, last_uid,
+							     TRUE);
 			mlist = g_slist_concat(mlist, newlist);
 		}
 	} else {
 		imap_delete_all_cached_messages(item);
-		mlist = imap_get_uncached_messages(session, item, 0, 0);
+		mlist = imap_get_uncached_messages(session, item, 0, 0, TRUE);
 		last_uid = procmsg_get_last_num_in_msg_list(mlist);
 	}
 
@@ -818,7 +957,6 @@ static GSList *imap_get_msg_list(Folder *folder, FolderItem *item,
 	item->last_num = last_uid;
 
 catch:
-	statusbar_pop_all();
 	return mlist;
 }
 
@@ -852,7 +990,6 @@ static gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint uid)
 
 	ok = imap_select(session, IMAP_FOLDER(folder), item->path,
 			 NULL, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		g_warning("can't select mailbox %s\n", item->path);
 		g_free(filename);
@@ -862,8 +999,6 @@ static gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint uid)
 	debug_print("getting message %d...\n", uid);
 	ok = imap_cmd_fetch(session, (guint32)uid, filename);
 
-	statusbar_pop_all();
-
 	if (ok != IMAP_SUCCESS) {
 		g_warning("can't fetch message %d\n", uid);
 		g_free(filename);
@@ -885,7 +1020,7 @@ static MsgInfo *imap_get_msginfo(Folder *folder, FolderItem *item, gint uid)
 	session = imap_session_get(folder);
 	g_return_val_if_fail(session != NULL, NULL);
 
-	list = imap_get_uncached_messages(session, item, uid, uid);
+	list = imap_get_uncached_messages(session, item, uid, uid, FALSE);
 	if (list) {
 		msginfo = (MsgInfo *)list->data;
 		list->data = NULL;
@@ -932,7 +1067,6 @@ static gint imap_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 
 	ok = imap_status(session, IMAP_FOLDER(folder), dest->path,
 			 &messages, &recent, &uid_next, &uid_validity, &unseen);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		g_warning("can't append messages\n");
 		return -1;
@@ -982,6 +1116,7 @@ static gint imap_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 
 		dest->last_num = last_uid;
 		dest->total++;
+		dest->updated = TRUE;
 
 		if (fileinfo->flags) {
 			if (MSG_IS_UNREAD(*fileinfo->flags))
@@ -1028,6 +1163,11 @@ static gint imap_do_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist,
 		return -1;
 	}
 
+	ok = imap_select(session, IMAP_FOLDER(folder), src->path,
+			 NULL, NULL, NULL, NULL);
+	if (ok != IMAP_SUCCESS)
+		return ok;
+
 	destdir = imap_get_real_path(IMAP_FOLDER(folder), dest->path);
 
 	seq_list = imap_get_seq_set_from_msglist(msglist);
@@ -1049,20 +1189,20 @@ static gint imap_do_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist,
 			imap_seq_set_free(seq_list);
 			return -1;
 		}
+	}
 
-		if (remove_source) {
-			ok = imap_set_message_flags
-				(session, seq_set, IMAP_FLAG_DELETED, TRUE);
-			if (ok == IMAP_SUCCESS)
-				ok = imap_cmd_expunge(session);
+	dest->updated = TRUE;
 
-			if (ok != IMAP_SUCCESS) {
-				imap_seq_set_free(seq_list);
-				return -1;
-			}
+	if (remove_source) {
+		imap_remove_msgs_by_seq_set(folder, src, seq_list);
+		if (ok != IMAP_SUCCESS) {
+			imap_seq_set_free(seq_list);
+			return ok;
 		}
 	}
 
+	imap_seq_set_free(seq_list);
+
 	for (cur = msglist; cur != NULL; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
 		dest->total++;
@@ -1077,13 +1217,11 @@ static gint imap_do_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist,
 				src->new--;
 			if (MSG_IS_UNREAD(msginfo->flags))
 				src->unread--;
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
 		}
 	}
 
-	imap_seq_set_free(seq_list);
-
 	g_free(destdir);
-	statusbar_pop_all();
 
 	if (ok == IMAP_SUCCESS)
 		return 0;
@@ -1170,6 +1308,39 @@ static gint imap_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 	return ret;
 }
 
+static gint imap_remove_msgs_by_seq_set(Folder *folder, FolderItem *item,
+					GSList *seq_list)
+{
+	gint ok;
+	IMAPSession *session;
+	GSList *cur;
+
+	g_return_val_if_fail(seq_list != NULL, -1);
+
+	session = imap_session_get(folder);
+	if (!session) return -1;
+
+	for (cur = seq_list; cur != NULL; cur = cur->next) {
+		gchar *seq_set = (gchar *)cur->data;
+
+		ok = imap_set_message_flags(session, seq_set, IMAP_FLAG_DELETED,
+					    TRUE);
+		if (ok != IMAP_SUCCESS) {
+			log_warning(_("can't set deleted flags: %s\n"),
+				    seq_set);
+			return ok;
+		}
+	}
+
+	ok = imap_cmd_expunge(session);
+	if (ok != IMAP_SUCCESS)
+		log_warning(_("can't expunge\n"));
+
+	item->updated = TRUE;
+
+	return ok;
+}
+
 static gint imap_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 {
 	GSList msglist;
@@ -1200,33 +1371,14 @@ static gint imap_remove_msgs(Folder *folder, FolderItem *item, GSList *msglist)
 
 	ok = imap_select(session, IMAP_FOLDER(folder), item->path,
 			 NULL, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS)
 		return ok;
 
 	seq_list = imap_get_seq_set_from_msglist(msglist);
-
-	for (cur = seq_list; cur != NULL; cur = cur->next) {
-		gchar *seq_set = (gchar *)cur->data;
-
-		ok = imap_set_message_flags(session, seq_set, IMAP_FLAG_DELETED,
-					    TRUE);
-		if (ok != IMAP_SUCCESS) {
-			log_warning(_("can't set deleted flags: %s\n"),
-				    seq_set);
-			imap_seq_set_free(seq_list);
-			return ok;
-		}
-	}
-
+	ok = imap_remove_msgs_by_seq_set(folder, item, seq_list);
 	imap_seq_set_free(seq_list);
-
-	ok = imap_cmd_expunge(session);
-	statusbar_pop_all();
-	if (ok != IMAP_SUCCESS) {
-		log_warning(_("can't expunge\n"));
+	if (ok != IMAP_SUCCESS)
 		return ok;
-	}
 
 	dir = folder_item_get_path(item);
 	dir_exist = is_dir_exist(dir);
@@ -1241,6 +1393,7 @@ static gint imap_remove_msgs(Folder *folder, FolderItem *item, GSList *msglist)
 			item->new--;
 		if (MSG_IS_UNREAD(msginfo->flags))
 			item->unread--;
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
 	}
 	g_free(dir);
 
@@ -1261,26 +1414,24 @@ static gint imap_remove_all_msg(Folder *folder, FolderItem *item)
 
 	ok = imap_select(session, IMAP_FOLDER(folder), item->path,
 			 NULL, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS)
 		return ok;
 
 	imap_cmd_gen_send(session, "STORE 1:* +FLAGS.SILENT (\\Deleted)");
 	ok = imap_cmd_ok(session, NULL);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		log_warning(_("can't set deleted flags: 1:*\n"));
 		return ok;
 	}
 
 	ok = imap_cmd_expunge(session);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		log_warning(_("can't expunge\n"));
 		return ok;
 	}
 
 	item->new = item->unread = item->total = 0;
+	item->updated = TRUE;
 
 	dir = folder_item_get_path(item);
 	if (is_dir_exist(dir))
@@ -1337,7 +1488,6 @@ static gint imap_scan_folder(Folder *folder, FolderItem *item)
 
 	ok = imap_status(session, IMAP_FOLDER(folder), item->path,
 			 &messages, &recent, &uid_next, &uid_validity, &unseen);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) return -1;
 
 	item->new = unseen > 0 ? recent : 0;
@@ -1345,6 +1495,7 @@ static gint imap_scan_folder(Folder *folder, FolderItem *item)
 	item->total = messages;
 	item->last_num = (messages > 0 && uid_next > 0) ? uid_next - 1 : 0;
 	/* item->mtime = uid_validity; */
+	item->updated = TRUE;
 
 	return 0;
 }
@@ -1633,7 +1784,6 @@ static GSList *imap_parse_list(IMAPSession *session, const gchar *real_path,
 	}
 
 	g_string_free(str, TRUE);
-	statusbar_pop_all();
 
 	return item_list;
 }
@@ -1706,7 +1856,7 @@ static FolderItem *imap_create_special_folder(Folder *folder,
 }
 
 static FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
-			       const gchar *name)
+				      const gchar *name)
 {
 	gchar *dirpath, *imap_path;
 	IMAPSession *session;
@@ -1756,7 +1906,6 @@ static FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 
 		argbuf = g_ptr_array_new();
 		ok = imap_cmd_list(session, NULL, imap_path, argbuf);
-		statusbar_pop_all();
 		if (ok != IMAP_SUCCESS) {
 			log_warning(_("can't create mailbox: LIST failed\n"));
 			g_free(imap_path);
@@ -1777,7 +1926,6 @@ static FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 
 		if (!exist) {
 			ok = imap_cmd_create(session, imap_path);
-			statusbar_pop_all();
 			if (ok != IMAP_SUCCESS) {
 				log_warning(_("can't create mailbox\n"));
 				g_free(imap_path);
@@ -1830,7 +1978,6 @@ static gint imap_rename_folder(Folder *folder, FolderItem *item,
 	session->mbox = NULL;
 	ok = imap_cmd_examine(session, "INBOX",
 			      &exists, &recent, &unseen, &uid_validity);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		g_free(real_oldpath);
 		return -1;
@@ -1848,7 +1995,6 @@ static gint imap_rename_folder(Folder *folder, FolderItem *item,
 	imap_path_separator_subst(real_newpath, separator);
 
 	ok = imap_cmd_rename(session, real_oldpath, real_newpath);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		log_warning(_("can't rename mailbox: %s to %s\n"),
 			    real_oldpath, real_newpath);
@@ -1905,14 +2051,12 @@ static gint imap_remove_folder(Folder *folder, FolderItem *item)
 
 	ok = imap_cmd_examine(session, "INBOX",
 			      &exists, &recent, &unseen, &uid_validity);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		g_free(path);
 		return -1;
 	}
 
 	ok = imap_cmd_delete(session, path);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS) {
 		log_warning(_("can't delete mailbox\n"));
 		g_free(path);
@@ -1931,7 +2075,8 @@ static gint imap_remove_folder(Folder *folder, FolderItem *item)
 
 static GSList *imap_get_uncached_messages(IMAPSession *session,
 					  FolderItem *item,
-					  guint32 first_uid, guint32 last_uid)
+					  guint32 first_uid, guint32 last_uid,
+					  gboolean update_count)
 {
 	gchar *tmp;
 	GSList *newlist = NULL;
@@ -1984,10 +2129,12 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 			log_warning(_("can't parse envelope: %s\n"), str->str);
 			continue;
 		}
-		if (MSG_IS_NEW(msginfo->flags))
-			item->new++;
-		if (MSG_IS_UNREAD(msginfo->flags))
-			item->unread++;
+		if (update_count) {
+			if (MSG_IS_NEW(msginfo->flags))
+				item->new++;
+			if (MSG_IS_UNREAD(msginfo->flags))
+				item->unread++;
+		}
 		if (item->stype == F_QUEUE) {
 			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_QUEUED);
 		} else if (item->stype == F_DRAFT) {
@@ -2003,7 +2150,8 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 			llast = llast->next;
 		}
 
-		item->total++;
+		if (update_count)
+			item->total++;
 	}
 
 	g_string_free(str, TRUE);
@@ -2118,7 +2266,7 @@ static SockInfo *imap_open(const gchar *server, gushort port)
 
 static GList *imap_parse_namespace_str(gchar *str)
 {
-	gchar *p = str;
+	guchar *p = str;
 	gchar *name;
 	gchar *separator;
 	IMAPNameSpace *namespace;
@@ -2300,7 +2448,7 @@ static gchar *imap_parse_atom(IMAPSession *session, gchar *src,
 	g_return_val_if_fail(str != NULL, cur_pos);
 
 	/* read the next line if the current response buffer is empty */
-	while (isspace(*cur_pos)) cur_pos++;
+	while (isspace(*(guchar *)cur_pos)) cur_pos++;
 	while (*cur_pos == '\0') {
 		if ((nextline = sock_getline(SESSION(session)->sock)) == NULL)
 			return cur_pos;
@@ -2311,7 +2459,7 @@ static gchar *imap_parse_atom(IMAPSession *session, gchar *src,
 		debug_print("IMAP4< %s\n", nextline);
 		g_free(nextline);
 
-		while (isspace(*cur_pos)) cur_pos++;
+		while (isspace(*(guchar *)cur_pos)) cur_pos++;
 	}
 
 	if (!strncmp(cur_pos, "NIL", 3)) {
@@ -2367,7 +2515,7 @@ static gchar *imap_get_header(IMAPSession *session, gchar *cur_pos,
 
 	g_return_val_if_fail(str != NULL, cur_pos);
 
-	while (isspace(*cur_pos)) cur_pos++;
+	while (isspace(*(guchar *)cur_pos)) cur_pos++;
 
 	g_return_val_if_fail(*cur_pos == '{', cur_pos);
 
@@ -2394,7 +2542,7 @@ static gchar *imap_get_header(IMAPSession *session, gchar *cur_pos,
 	*headers = g_strndup(cur_pos, len);
 	cur_pos += len;
 
-	while (isspace(*cur_pos)) cur_pos++;
+	while (isspace(*(guchar *)cur_pos)) cur_pos++;
 	while (*cur_pos == '\0') {
 		if ((nextline = sock_getline(SESSION(session)->sock)) == NULL)
 			return cur_pos;
@@ -2404,7 +2552,7 @@ static gchar *imap_get_header(IMAPSession *session, gchar *cur_pos,
 		debug_print("IMAP4< %s\n", nextline);
 		g_free(nextline);
 
-		while (isspace(*cur_pos)) cur_pos++;
+		while (isspace(*(guchar *)cur_pos)) cur_pos++;
 	}
 
 	return cur_pos;
@@ -2436,6 +2584,28 @@ static MsgFlags imap_parse_flags(const gchar *flag_str)
 	return flags;
 }
 
+static IMAPFlags imap_parse_imap_flags(const gchar *flag_str)  
+{
+	const gchar *p = flag_str;
+	IMAPFlags flags = 0;
+
+	while ((p = strchr(p, '\\')) != NULL) {
+		p++;
+
+		if (g_strncasecmp(p, "Seen", 4) == 0) {
+			flags |= IMAP_FLAG_SEEN;
+		} else if (g_strncasecmp(p, "Deleted", 7) == 0) {
+			flags |= IMAP_FLAG_DELETED;
+		} else if (g_strncasecmp(p, "Flagged", 7) == 0) {
+			flags |= IMAP_FLAG_FLAGGED;
+		} else if (g_strncasecmp(p, "Answered", 8) == 0) {
+			flags |= IMAP_FLAG_ANSWERED;
+		}
+	}
+
+	return flags;
+}
+
 static MsgInfo *imap_parse_envelope(IMAPSession *session, FolderItem *item,
 				    GString *line_str)
 {
@@ -2512,6 +2682,8 @@ static MsgInfo *imap_parse_envelope(IMAPSession *session, FolderItem *item,
 		}
 	}
 
+#undef PARSE_ONE_ELEMENT
+
 	if (msginfo) {
 		msginfo->msgnum = uid;
 		msginfo->size = size;
@@ -2549,7 +2721,6 @@ static gint imap_msg_list_change_perm_flags(GSList *msglist, MsgPermFlags flags,
 
 	ok = imap_select(session, IMAP_FOLDER(folder), msginfo->folder->path,
 			 NULL, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != IMAP_SUCCESS)
 		return ok;
 
@@ -3612,7 +3783,7 @@ static gchar *imap_locale_to_modified_utf7(const gchar *from)
 			norm_utf7_len -= 2;
 			from_tmp++;
 			from_len--;
-		} else if (IS_PRINT(*from_tmp)) {
+		} else if (IS_PRINT(*(guchar *)from_tmp)) {
 			/* printable ascii char */
 			*norm_utf7_p = *from_tmp;
 			norm_utf7_p++;
@@ -3624,7 +3795,7 @@ static gchar *imap_locale_to_modified_utf7(const gchar *from)
 
 			/* unprintable char: convert to UTF-7 */
 			p = from_tmp;
-			while (!IS_PRINT(*p) && conv_len < from_len) {
+			while (!IS_PRINT(*(guchar *)p) && conv_len < from_len) {
 				mb_len = mblen(p, MB_LEN_MAX);
 				if (mb_len <= 0) {
 					g_warning("wrong multibyte sequence\n");
