@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -79,8 +80,8 @@ static GdkBitmap *okxpmmask;
 
 static void inc_finished		(MainWindow		*mainwin,
 					 gboolean		 new_messages);
-static gint inc_account_mail		(PrefsAccount		*account,
-					 MainWindow		*mainwin);
+static gint inc_account_mail_real	(MainWindow		*mainwin,
+					 PrefsAccount		*account);
 
 static IncProgressDialog *inc_progress_dialog_create
 					(gboolean		 autocheck);
@@ -92,9 +93,25 @@ static void inc_session_destroy		(IncSession		*session);
 static gint inc_start			(IncProgressDialog	*inc_dialog);
 static IncState inc_pop3_session_do	(IncSession		*session);
 
+static void inc_progress_dialog_update	(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
+
 static void inc_progress_dialog_set_label
 					(IncProgressDialog	*inc_dialog,
 					 IncSession		*inc_session);
+static void inc_progress_dialog_set_progress
+					(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
+
+static void inc_update_folderview	(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
+
+static void inc_progress_dialog_update_periodic
+					(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
+static void inc_update_folderview_periodic
+					(IncProgressDialog	*inc_dialog,
+					 IncSession		*inc_session);
 
 static gint inc_recv_data_progressive	(Session	*session,
 					 guint		 cur_len,
@@ -106,6 +123,8 @@ static gint inc_recv_data_finished	(Session	*session,
 static gint inc_recv_message		(Session	*session,
 					 const gchar	*msg,
 					 gpointer	 data);
+static gint inc_drop_message		(Pop3Session	*session,
+					 const gchar	*file);
 
 static void inc_put_error		(IncState	 istate,
 					 const gchar	*msg);
@@ -159,6 +178,9 @@ void inc_mail(MainWindow *mainwin)
 
 	if (inc_lock_count) return;
 
+	if (!main_window_toggle_online_if_offline(mainwin))
+		return;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
 	main_window_lock(mainwin);
@@ -173,9 +195,6 @@ void inc_mail(MainWindow *mainwin)
 
 		if (prefs_common.inc_local)
 			new_msgs = inc_spool();
-
-		if (new_msgs <= 0)
-			new_msgs = 1;
 	} else {
 		if (prefs_common.inc_local) {
 			new_msgs = inc_spool();
@@ -183,7 +202,7 @@ void inc_mail(MainWindow *mainwin)
 				new_msgs = 0;
 		}
 
-		new_msgs += inc_account_mail(cur_account, mainwin);
+		new_msgs += inc_account_mail_real(mainwin, cur_account);
 	}
 
 	inc_finished(mainwin, new_msgs > 0);
@@ -191,7 +210,7 @@ void inc_mail(MainWindow *mainwin)
 	inc_autocheck_timer_set();
 }
 
-static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
+static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
@@ -220,6 +239,28 @@ static gint inc_account_mail(PrefsAccount *account, MainWindow *mainwin)
 	return inc_start(inc_dialog);
 }
 
+gint inc_account_mail(MainWindow *mainwin, PrefsAccount *account)
+{
+	gint new_msgs;
+
+	if (inc_lock_count) return 0;
+
+	if (!main_window_toggle_online_if_offline(mainwin))
+		return 0;
+
+	inc_autocheck_timer_remove();
+	summary_write_cache(mainwin->summaryview);
+	main_window_lock(mainwin);
+
+	new_msgs = inc_account_mail_real(mainwin, account);
+
+	inc_finished(mainwin, new_msgs > 0);
+	main_window_unlock(mainwin);
+	inc_autocheck_timer_set();
+
+	return new_msgs;
+}
+
 void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 {
 	GList *list, *queue_list = NULL;
@@ -228,6 +269,9 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 
 	if (inc_lock_count) return;
 
+	if (!main_window_toggle_online_if_offline(mainwin))
+		return;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
 	main_window_lock(mainwin);
@@ -238,16 +282,8 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 			new_msgs = 0;
 	}
 
-	list = account_get_list();
-	if (!list) {
-		inc_finished(mainwin, new_msgs > 0);
-		main_window_unlock(mainwin);
-		inc_autocheck_timer_set();
-		return;
-	}
-
-	/* check IMAP4 folders */
-	for (; list != NULL; list = list->next) {
+	/* check IMAP4 / News folders */
+	for (list = account_get_list(); list != NULL; list = list->next) {
 		PrefsAccount *account = list->data;
 		if ((account->protocol == A_IMAP4 ||
 		     account->protocol == A_NNTP) && account->recv_at_getall) {
@@ -272,23 +308,18 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 		}
 	}
 
-	if (!queue_list) {
-		inc_finished(mainwin, new_msgs > 0);
-		main_window_unlock(mainwin);
-		inc_autocheck_timer_set();
-		return;
+	if (queue_list) {
+		inc_dialog = inc_progress_dialog_create(autocheck);
+		inc_dialog->queue_list = queue_list;
+		inc_dialog->mainwin = mainwin;
+		inc_progress_dialog_set_list(inc_dialog);
+
+		main_window_set_toolbar_sensitive(mainwin);
+		main_window_set_menu_sensitive(mainwin);
+
+		new_msgs += inc_start(inc_dialog);
 	}
 
-	inc_dialog = inc_progress_dialog_create(autocheck);
-	inc_dialog->queue_list = queue_list;
-	inc_dialog->mainwin = mainwin;
-	inc_progress_dialog_set_list(inc_dialog);
-
-	main_window_set_toolbar_sensitive(mainwin);
-	main_window_set_menu_sensitive(mainwin);
-
-	new_msgs += inc_start(inc_dialog);
-
 	inc_finished(mainwin, new_msgs > 0);
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
@@ -327,6 +358,8 @@ static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 	}
 
 	dialog->dialog = progress;
+	gettimeofday(&dialog->progress_tv, NULL);
+	gettimeofday(&dialog->folder_tv, NULL);
 	dialog->queue_list = NULL;
 	dialog->cur_row = 0;
 
@@ -357,10 +390,7 @@ static void inc_progress_dialog_clear(IncProgressDialog *inc_dialog)
 {
 	progress_dialog_set_value(inc_dialog->dialog, 0.0);
 	progress_dialog_set_label(inc_dialog->dialog, "");
-	gtk_progress_set_show_text
-		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), FALSE);
-	gtk_progress_bar_update
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
+	main_window_progress_off(inc_dialog->mainwin);
 }
 
 static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
@@ -369,10 +399,7 @@ static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
 
 	inc_dialog_list = g_list_remove(inc_dialog_list, inc_dialog);
 
-	gtk_progress_set_show_text
-		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), FALSE);
-	gtk_progress_bar_update
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar), 0.0);
+	main_window_progress_off(inc_dialog->mainwin);
 	progress_dialog_destroy(inc_dialog->dialog);
 
 	g_free(inc_dialog);
@@ -390,9 +417,20 @@ static IncSession *inc_session_new(PrefsAccount *account)
 		return NULL;
 
 	session = g_new0(IncSession, 1);
+
 	session->session = pop3_session_new(account);
 	session->session->data = session;
+	POP3_SESSION(session->session)->drop_message = inc_drop_message;
+	session_set_recv_message_notify(session->session,
+					inc_recv_message, session);
+	session_set_recv_data_progressive_notify(session->session,
+						 inc_recv_data_progressive,
+						 session);
+	session_set_recv_data_notify(session->session,
+				     inc_recv_data_finished, session);
+
 	session->folder_table = g_hash_table_new(NULL, NULL);
+	session->tmp_folder_table = g_hash_table_new(NULL, NULL);
 
 	return session;
 }
@@ -403,6 +441,7 @@ static void inc_session_destroy(IncSession *session)
 
 	session_destroy(session->session);
 	g_hash_table_destroy(session->folder_table);
+	g_hash_table_destroy(session->tmp_folder_table);
 	g_free(session);
 }
 
@@ -482,13 +521,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		SET_PIXMAP_AND_TEXT(currentxpm, currentxpmmask,
 				    _("Retrieving"));
 
-		session_set_recv_message_notify(session->session,
-						inc_recv_message, session);
-		session_set_recv_data_progressive_notify
-			(session->session, inc_recv_data_progressive, session);
-		session_set_recv_data_notify(session->session,
-					     inc_recv_data_finished, session);
-
 		/* begin POP3 session */
 		inc_state = inc_pop3_session_do(session);
 
@@ -523,6 +555,9 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 		case INC_EOF:
 			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask, _("Error"));
 			break;
+		case INC_TIMEOUT:
+			SET_PIXMAP_AND_TEXT(errorxpm, errorxpmmask, _("Timeout"));
+			break;
 		case INC_CANCEL:
 			SET_PIXMAP_AND_TEXT(okxpm, okxpmmask, _("Cancelled"));
 			break;
@@ -530,8 +565,6 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			break;
 		}
 
-		statusbar_pop_all();
-
 		new_msgs += pop3_session->cur_total_num;
 
 		if (!prefs_common.scan_all_after_inc) {
@@ -599,7 +632,7 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			g_list_remove(inc_dialog->queue_list, session);
 	}
 
-	if (prefs_common.close_recv_dialog)
+	if (prefs_common.close_recv_dialog || !inc_dialog->show_dialog)
 		inc_progress_dialog_destroy(inc_dialog);
 	else {
 		gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window),
@@ -648,6 +681,9 @@ static IncState inc_pop3_session_do(IncSession *session)
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
 
+	session_set_timeout(SESSION(pop3_session),
+			    prefs_common.io_timeout_secs * 1000);
+
 	if (session_connect(SESSION(pop3_session), server, port) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    server, port);
@@ -660,20 +696,26 @@ static IncState inc_pop3_session_do(IncSession *session)
 	       session->inc_state != INC_CANCEL)
 		gtk_main_iteration();
 
-	statusbar_pop_all();
-
 	if (session->inc_state == INC_SUCCESS) {
 		switch (pop3_session->error_val) {
 		case PS_SUCCESS:
-			if (SESSION(pop3_session)->state == SESSION_ERROR) {
+			switch (SESSION(pop3_session)->state) {
+			case SESSION_ERROR:
 				if (pop3_session->state == POP3_READY)
 					session->inc_state = INC_CONNECT_ERROR;
 				else
 					session->inc_state = INC_ERROR;
-			} else if (SESSION(pop3_session)->state == SESSION_EOF)
+				break;
+			case SESSION_EOF:
 				session->inc_state = INC_EOF;
-			else
+				break;
+			case SESSION_TIMEOUT:
+				session->inc_state = INC_TIMEOUT;
+				break;
+			default:
 				session->inc_state = INC_SUCCESS;
+				break;
+			}
 			break;
 		case PS_AUTHFAIL:
 			session->inc_state = INC_AUTH_FAILED;
@@ -693,13 +735,22 @@ static IncState inc_pop3_session_do(IncSession *session)
 		}
 	}
 
+	session_disconnect(SESSION(pop3_session));
+	statusbar_pop_all();
+
 	return session->inc_state;
 }
 
+static void inc_progress_dialog_update(IncProgressDialog *inc_dialog,
+				       IncSession *inc_session)
+{
+	inc_progress_dialog_set_label(inc_dialog, inc_session);
+	inc_progress_dialog_set_progress(inc_dialog, inc_session);
+}
+
 static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 					  IncSession *inc_session)
 {
-	gchar buf[MSGBUFSIZE];
 	ProgressDialog *dialog = inc_dialog->dialog;
 	Pop3Session *session;
 
@@ -734,24 +785,18 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 			(dialog, _("Getting the size of messages (LIST)..."));
 		break;
 	case POP3_RETR:
-		gtk_progress_set_show_text
-			(GTK_PROGRESS(inc_dialog->mainwin->progressbar), TRUE);
-		g_snprintf(buf, sizeof(buf), "%d / %d",
-			   session->cur_msg, session->count);
-		gtk_progress_set_format_string
-			(GTK_PROGRESS(inc_dialog->mainwin->progressbar), buf);
-		inc_recv_data_progressive
-			(SESSION(session), 0,
-			 session->msg[session->cur_msg].size,
-			 inc_session);
+	case POP3_RETR_RECV:
 		break;
 	case POP3_DELETE:
+#if 0
 		if (session->msg[session->cur_msg].recv_time <
 			session->current_time) {
+			gchar buf[MSGBUFSIZE];
 			g_snprintf(buf, sizeof(buf), _("Deleting message %d"),
 				   session->cur_msg);
 			progress_dialog_set_label(dialog, buf);
 		}
+#endif
 		break;
 	case POP3_LOGOUT:
 		progress_dialog_set_label(dialog, _("Quitting"));
@@ -761,16 +806,125 @@ static void inc_progress_dialog_set_label(IncProgressDialog *inc_dialog,
 	}
 }
 
+static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
+					     IncSession *inc_session)
+{
+	gchar buf[MSGBUFSIZE];
+	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
+	gchar *total_size_str;
+	gint cur_total;
+	gint total;
+
+	if (!pop3_session->new_msg_exist) return;
+
+	cur_total = inc_session->cur_total_bytes;
+	total = pop3_session->total_bytes;
+	if (pop3_session->state == POP3_RETR ||
+	    pop3_session->state == POP3_RETR_RECV ||
+	    pop3_session->state == POP3_DELETE) {
+		Xstrdup_a(total_size_str, to_human_readable(total), return);
+		g_snprintf(buf, sizeof(buf),
+			   _("Retrieving message (%d / %d) (%s / %s)"),
+			   pop3_session->cur_msg, pop3_session->count,
+			   to_human_readable(cur_total), total_size_str);
+		progress_dialog_set_label(inc_dialog->dialog, buf);
+	}
+
+	progress_dialog_set_percentage
+		(inc_dialog->dialog,(gfloat)cur_total / (gfloat)total);
+
+	gtk_progress_set_show_text
+		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), TRUE);
+	g_snprintf(buf, sizeof(buf), "%d / %d",
+		   pop3_session->cur_msg, pop3_session->count);
+	gtk_progress_set_format_string
+		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), buf);
+	gtk_progress_bar_update
+		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
+		 (gfloat)cur_total / (gfloat)total);
+
+	if (pop3_session->cur_total_num > 0) {
+		g_snprintf(buf, sizeof(buf),
+			   _("Retrieving (%d message(s) (%s) received)"),
+			   pop3_session->cur_total_num,
+			   to_human_readable
+			   (pop3_session->cur_total_recv_bytes));
+		gtk_clist_set_text(GTK_CLIST(inc_dialog->dialog->clist),
+				   inc_dialog->cur_row, 2, buf);
+	}
+}
+
+static gboolean hash_remove_func(gpointer key, gpointer value, gpointer data)
+{
+	return TRUE;
+}
+
+static void inc_update_folderview(IncProgressDialog *inc_dialog,
+				  IncSession *inc_session)
+{
+	if (g_hash_table_size(inc_session->tmp_folder_table) > 0) {
+		folderview_update_item_foreach(inc_session->tmp_folder_table,
+					       FALSE);
+		g_hash_table_foreach_remove(inc_session->tmp_folder_table,
+					    hash_remove_func, NULL);
+	}
+}
+
+static void inc_progress_dialog_update_periodic(IncProgressDialog *inc_dialog,
+						IncSession *inc_session)
+{
+	struct timeval tv_cur;
+	struct timeval tv_result;
+	gint msec;
+
+	gettimeofday(&tv_cur, NULL);
+
+	tv_result.tv_sec = tv_cur.tv_sec - inc_dialog->progress_tv.tv_sec;
+	tv_result.tv_usec = tv_cur.tv_usec - inc_dialog->progress_tv.tv_usec;
+	if (tv_result.tv_usec < 0) {
+		tv_result.tv_sec--;
+		tv_result.tv_usec += 1000000;
+	}
+
+	msec = tv_result.tv_sec * 1000 + tv_result.tv_usec / 1000;
+	if (msec > PROGRESS_UPDATE_INTERVAL) {
+		inc_progress_dialog_update(inc_dialog, inc_session);
+		inc_dialog->progress_tv.tv_sec = tv_cur.tv_sec;
+		inc_dialog->progress_tv.tv_usec = tv_cur.tv_usec;
+	}
+}
+
+static void inc_update_folderview_periodic(IncProgressDialog *inc_dialog,
+					   IncSession *inc_session)
+{
+	struct timeval tv_cur;
+	struct timeval tv_result;
+	gint msec;
+
+	gettimeofday(&tv_cur, NULL);
+
+	tv_result.tv_sec = tv_cur.tv_sec - inc_dialog->folder_tv.tv_sec;
+	tv_result.tv_usec = tv_cur.tv_usec - inc_dialog->folder_tv.tv_usec;
+	if (tv_result.tv_usec < 0) {
+		tv_result.tv_sec--;
+		tv_result.tv_usec += 1000000;
+	}
+
+	msec = tv_result.tv_sec * 1000 + tv_result.tv_usec / 1000;
+	if (msec > FOLDER_UPDATE_INTERVAL) {
+		inc_update_folderview(inc_dialog, inc_session);
+		inc_dialog->folder_tv.tv_sec = tv_cur.tv_sec;
+		inc_dialog->folder_tv.tv_usec = tv_cur.tv_usec;
+	}
+}
+
 static gint inc_recv_data_progressive(Session *session, guint cur_len,
 				      guint total_len, gpointer data)
 {
-	gchar buf[MSGBUFSIZE];
 	IncSession *inc_session = (IncSession *)data;
 	Pop3Session *pop3_session = POP3_SESSION(session);
 	IncProgressDialog *inc_dialog;
-	ProgressDialog *dialog;
 	gint cur_total;
-	gchar *total_size;
 
 	g_return_val_if_fail(inc_session != NULL, -1);
 
@@ -781,25 +935,14 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 
 	if (!pop3_session->new_msg_exist) return 0;
 
-	inc_dialog = (IncProgressDialog *)inc_session->data;
-	dialog = inc_dialog->dialog;
-
 	cur_total = pop3_session->cur_total_bytes + cur_len;
 	if (cur_total > pop3_session->total_bytes)
 		cur_total = pop3_session->total_bytes;
+	inc_session->cur_total_bytes = cur_total;
 
-	Xstrdup_a(total_size, to_human_readable(pop3_session->total_bytes),
-		  return FALSE);
-	g_snprintf(buf, sizeof(buf),
-		   _("Retrieving message (%d / %d) (%s / %s)"),
-		   pop3_session->cur_msg, pop3_session->count,
-		   to_human_readable(cur_total), total_size);
-	progress_dialog_set_label(dialog, buf);
-	progress_dialog_set_percentage
-		(dialog, (gfloat)cur_total / (gfloat)pop3_session->total_bytes);
-	gtk_progress_bar_update
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
-		 (gfloat)cur_total / (gfloat)pop3_session->total_bytes);
+	inc_dialog = (IncProgressDialog *)inc_session->data;
+	inc_progress_dialog_update_periodic(inc_dialog, inc_session);
+	inc_update_folderview_periodic(inc_dialog, inc_session);
 
 	return 0;
 }
@@ -807,24 +950,18 @@ static gint inc_recv_data_progressive(Session *session, guint cur_len,
 static gint inc_recv_data_finished(Session *session, guint len, gpointer data)
 {
 	IncSession *inc_session = (IncSession *)data;
-	Pop3Session *pop3_session = POP3_SESSION(session);
 	IncProgressDialog *inc_dialog;
-	gchar *msg;
 
 	g_return_val_if_fail(inc_session != NULL, -1);
 
 	inc_dialog = (IncProgressDialog *)inc_session->data;
-	inc_recv_data_progressive(session, 0, len, inc_session);
-	inc_progress_dialog_set_label(inc_dialog, inc_session);
 
-	if (pop3_session->cur_total_num == 0) return 0;
+	inc_recv_data_progressive(session, 0, 0, inc_session);
 
-	msg = g_strdup_printf(_("Retrieving (%d message(s) (%s) received)"),
-			      pop3_session->cur_total_num,
-			      to_human_readable
-				(pop3_session->cur_total_recv_bytes));
-	gtk_clist_set_text(GTK_CLIST(inc_dialog->dialog->clist),
-			   inc_dialog->cur_row, 2, msg);
+	if (POP3_SESSION(session)->state == POP3_LOGOUT) {
+		inc_progress_dialog_update(inc_dialog, inc_session);
+		inc_update_folderview(inc_dialog, inc_session);
+	}
 
 	return 0;
 }
@@ -837,18 +974,40 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 	g_return_val_if_fail(inc_session != NULL, -1);
 
 	inc_dialog = (IncProgressDialog *)inc_session->data;
-	inc_progress_dialog_set_label(inc_dialog, inc_session);
+
+	switch (POP3_SESSION(session)->state) {
+	case POP3_GETAUTH_USER:
+	case POP3_GETAUTH_PASS:
+	case POP3_GETAUTH_APOP:
+	case POP3_GETRANGE_STAT:
+	case POP3_GETRANGE_LAST:
+	case POP3_GETRANGE_UIDL:
+	case POP3_GETSIZE_LIST:
+		inc_progress_dialog_update(inc_dialog, inc_session);
+		break;
+	case POP3_RETR:
+		inc_recv_data_progressive(session, 0, 0, inc_session);
+		break;
+	case POP3_LOGOUT:
+		inc_progress_dialog_update(inc_dialog, inc_session);
+		inc_update_folderview(inc_dialog, inc_session);
+		break;
+	default:
+		break;
+	}
 
 	return 0;
 }
 
-gint inc_drop_message(const gchar *file, Pop3Session *session)
+static gint inc_drop_message(Pop3Session *session, const gchar *file)
 {
 	FolderItem *inbox;
 	FolderItem *dropfolder;
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
 
+	g_return_val_if_fail(inc_session != NULL, -1);
+
 	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
 			(session->ac_prefs->inbox);
@@ -866,7 +1025,6 @@ gint inc_drop_message(const gchar *file, Pop3Session *session)
 			filter_get_dest_folder(prefs_common.fltlist, file);
 		if (!dropfolder) dropfolder = inbox;
 		else if (!strcmp(dropfolder->path, FILTER_NOT_RECEIVE)) {
-			debug_print(_("a message won't be received\n"));
 			unlink(file);
 			return 1;
 		}
@@ -883,6 +1041,8 @@ gint inc_drop_message(const gchar *file, Pop3Session *session)
 		g_hash_table_insert(inc_session->folder_table, dropfolder,
 				    GINT_TO_POINTER(1));
 	}
+	g_hash_table_insert(inc_session->tmp_folder_table, dropfolder,
+			    GINT_TO_POINTER(1));
 
 	if (folder_item_add_msg(dropfolder, file, NULL, TRUE) < 0) {
 		unlink(file);
@@ -958,6 +1118,12 @@ static void inc_put_error(IncState istate, const gchar *msg)
 		else
 			err_msg = g_strdup(log_msg);
 		break;
+	case INC_TIMEOUT:
+		log_msg = _("Session timed out.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		err_msg = g_strdup(log_msg);
+		break;
 	default:
 		break;
 	}
@@ -1055,8 +1221,8 @@ static gint get_spool(FolderItem *dest, const gchar *mbox)
 	if ((lockfd = lock_mbox(mbox, LOCK_FLOCK)) < 0)
 		return -1;
 
-	g_snprintf(tmp_mbox, sizeof(tmp_mbox), "%s%ctmpmbox.%08x",
-		   get_tmp_dir(), G_DIR_SEPARATOR, (gint)mbox);
+	g_snprintf(tmp_mbox, sizeof(tmp_mbox), "%s%ctmpmbox.%p",
+		   get_tmp_dir(), G_DIR_SEPARATOR, mbox);
 
 	if (copy_mbox(mbox, tmp_mbox) < 0) {
 		unlock_mbox(mbox, lockfd, LOCK_FLOCK);
