@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,9 @@
 #include "utils.h"
 #include "gtkutils.h"
 
+#define MAX_LINES	500
+#define TRIM_LINES	25
+
 static LogWindow *logwindow;
 
 static void hide_cb	(GtkWidget	*widget,
@@ -83,6 +86,7 @@ LogWindow *log_window_create(void)
 	logwin->window = window;
 	logwin->scrolledwin = scrolledwin;
 	logwin->text = text;
+	logwin->lines = 0;
 
 	logwindow = logwin;
 
@@ -159,8 +163,44 @@ void log_window_append(const gchar *str, LogType type)
 		break;
 	}
 
+	if (logwindow->lines == MAX_LINES) {
+		guint len;
+		guint text_len;
+		gint lines = 0;
+		gboolean frozen;
+
+		text_len = gtk_text_get_length(text);
+		for (len = 0; len < text_len; len++) {
+			if (GTK_TEXT_INDEX(text, len) == '\n') {
+				lines++;
+				if (lines == TRIM_LINES ||
+				    lines == logwindow->lines) {
+					len++;
+					break;
+				}
+			}
+		}
+
+		gtk_text_freeze(text);
+		gtk_text_set_point(text, 0);
+		gtk_text_thaw(text);
+
+		/* gtk_text_forward_delete() crashes without this */
+		frozen = text->freeze_count > 0;
+		if (frozen)
+			gtk_text_thaw(text);
+		gtk_text_forward_delete(text, len);
+		if (frozen)
+			gtk_text_freeze(text);
+
+		gtk_text_set_point(text, gtk_text_get_length(text));
+		logwindow->lines -= lines;
+	}
+
 	if (head) gtk_text_insert(text, NULL, color, NULL, head, -1);
 	gtk_text_insert(text, NULL, color, NULL, str, -1);
+
+	logwindow->lines++;
 }
 
 static void hide_cb(GtkWidget *widget, LogWindow *logwin)
