@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include <gtk/gtkhandlebox.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtktooltips.h>
 #include <string.h>
 
 #include "intl.h"
@@ -144,6 +145,16 @@ static void toolbar_account_button_pressed	(GtkWidget	*widget,
 						 GdkEventButton	*event,
 						 gpointer	 data);
 #endif
+
+static void toolbar_child_attached		(GtkWidget	*widget,
+						 GtkWidget	*child,
+						 gpointer	 data);
+static void toolbar_child_detached		(GtkWidget	*widget,
+						 GtkWidget	*child,
+						 gpointer	 data);
+
+static void online_switch_clicked		(GtkWidget	*widget,
+						 gpointer	 data);
 static void ac_label_button_pressed		(GtkWidget	*widget,
 						 GdkEventButton	*event,
 						 gpointer	 data);
@@ -201,6 +212,9 @@ static void save_as_cb		 (MainWindow	*mainwin,
 static void print_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
+static void toggle_offline_cb	 (MainWindow	*mainwin,
+				  guint		 action,
+				  GtkWidget	*widget);
 static void app_exit_cb		 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
@@ -265,10 +279,6 @@ static void show_all_header_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
 
-static void reedit_cb			(MainWindow	*mainwin,
-					 guint		 action,
-					 GtkWidget	*widget);
-
 static void move_to_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -295,6 +305,11 @@ static void mark_as_read_cb		(MainWindow	*mainwin,
 static void mark_all_read_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
+
+static void reedit_cb			(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void add_address_cb		(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -409,8 +424,10 @@ static void new_account_cb	 (MainWindow	*mainwin,
 				  guint		 action,
 				  GtkWidget	*widget);
 
-static void account_menu_cb	 (GtkMenuItem	*menuitem,
-				  gpointer	 data);
+static void account_selector_menu_cb	 (GtkMenuItem	*menuitem,
+					  gpointer	 data);
+static void account_receive_menu_cb	 (GtkMenuItem	*menuitem,
+					  gpointer	 data);
 
 static void manual_open_cb	 (MainWindow	*mainwin,
 				  guint		 action,
@@ -442,6 +459,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_File/_Save as..."),		"<control>S", save_as_cb, 0, NULL},
 	{N_("/_File/_Print..."),		NULL, print_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_File/_Work offline"),		NULL, toggle_offline_cb, 0, "<ToggleItem>"},
+	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	/* {N_("/_File/_Close"),		"<alt>W", app_exit_cb, 0, NULL}, */
 	{N_("/_File/E_xit"),			"<control>Q", app_exit_cb, 0, NULL},
 
@@ -568,6 +587,8 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	 CODESET_ACTION(C_ISO_8859_5)},
 	{N_("/_View/_Code set/Cyrillic (KOI8-_R)"),
 	 CODESET_ACTION(C_KOI8_R)},
+	{N_("/_View/_Code set/Cyrillic (KOI8-U)"),
+	 CODESET_ACTION(C_KOI8_U)},
 	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
 	 CODESET_ACTION(C_CP1251)},
 	CODESET_SEPARATOR,
@@ -615,11 +636,14 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Update summary"),		"<control><alt>U", update_summary_cb,  0, NULL},
 
 	{N_("/_Message"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Message/Get new ma_il"),		"<control>I",	inc_mail_cb, 0, NULL},
-	{N_("/_Message/Get from _all accounts"),
+	{N_("/_Message/Recei_ve"),		NULL, NULL, 0, "<Branch>"},
+	{N_("/_Message/Recei_ve/Get from _current account"),
+						"<control>I",	inc_mail_cb, 0, NULL},
+	{N_("/_Message/Recei_ve/Get from _all accounts"),
 						"<shift><control>I", inc_all_account_mail_cb, 0, NULL},
-	{N_("/_Message/Cancel receivin_g"),	NULL, inc_cancel_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Recei_ve/Cancel receivin_g"),
+						NULL, inc_cancel_cb, 0, NULL},
+	{N_("/_Message/Recei_ve/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/_Send queued messages"), NULL, send_queue_cb, 0, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/Compose _new message"),	"<control>M",	compose_cb, 0, NULL},
@@ -636,8 +660,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						"<shift><control><alt>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/Redirec_t"),		NULL, reply_cb, COMPOSE_REDIRECT, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Re-_edit"),		NULL, reedit_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/M_ove..."),		"<control>O", move_to_cb, 0, NULL},
 	{N_("/_Message/_Copy..."),		"<shift><control>O", copy_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<control>D", delete_cb,  0, NULL},
@@ -650,13 +672,18 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Mark/Mark as rea_d"),
 						NULL, mark_as_read_cb, 0, NULL},
 	{N_("/_Message/_Mark/Mark all _read"),	NULL, mark_all_read_cb, 0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Re-_edit"),		NULL, reedit_cb, 0, NULL},
 
 	{N_("/_Tools"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Address book"),		"<shift><control>A", addressbook_open_cb, 0, NULL},
 	{N_("/_Tools/Add sender to address boo_k"),
 						NULL, add_address_cb, 0, NULL},
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Tools/_Filter messages"),		NULL, filter_cb, 0, NULL},
+	{N_("/_Tools/_Filter all messages in folder"),
+						NULL, filter_cb, 0, NULL},
+	{N_("/_Tools/Filter _selected messages"),
+						NULL, filter_cb, 1, NULL},
 	{N_("/_Tools/_Create filter rule"),	NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Create filter rule/_Automatically"),
 						NULL, create_filter_cb, FILTER_BY_AUTO, NULL},
@@ -719,6 +746,11 @@ MainWindow *main_window_create(SeparateType type)
 	GtkWidget *statusbar;
 	GtkWidget *progressbar;
 	GtkWidget *statuslabel;
+	GtkWidget *online_hbox;
+	GtkWidget *online_switch;
+	GtkWidget *online_pixmap;
+	GtkWidget *offline_pixmap;
+	GtkTooltips *online_tip;
 	GtkWidget *ac_button;
 	GtkWidget *ac_label;
 
@@ -772,6 +804,10 @@ MainWindow *main_window_create(SeparateType type)
 	handlebox = gtk_handle_box_new();
 	gtk_widget_show(handlebox);
 	gtk_box_pack_start(GTK_BOX(vbox), handlebox, FALSE, FALSE, 0);
+	gtk_signal_connect(GTK_OBJECT(handlebox), "child_attached",
+			   GTK_SIGNAL_FUNC(toolbar_child_attached), mainwin);
+	gtk_signal_connect(GTK_OBJECT(handlebox), "child_detached",
+			   GTK_SIGNAL_FUNC(toolbar_child_detached), mainwin);
 
 	main_window_toolbar_create(mainwin, handlebox);
 
@@ -794,6 +830,25 @@ MainWindow *main_window_create(SeparateType type)
 	statuslabel = gtk_label_new("");
 	gtk_box_pack_start(GTK_BOX(hbox_stat), statuslabel, FALSE, FALSE, 0);
 
+	online_hbox = gtk_hbox_new(FALSE, 0);
+
+	online_pixmap = stock_pixmap_widget(hbox_stat, STOCK_PIXMAP_ONLINE);
+	offline_pixmap = stock_pixmap_widget(hbox_stat, STOCK_PIXMAP_OFFLINE);
+	gtk_box_pack_start(GTK_BOX(online_hbox), online_pixmap,
+			   FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(online_hbox), offline_pixmap,
+			   FALSE, FALSE, 0);
+
+	online_switch = gtk_button_new();
+	gtk_button_set_relief(GTK_BUTTON(online_switch), GTK_RELIEF_NONE);
+	GTK_WIDGET_UNSET_FLAGS(online_switch, GTK_CAN_FOCUS);
+	gtk_container_add(GTK_CONTAINER(online_switch), online_hbox);
+	gtk_signal_connect(GTK_OBJECT(online_switch), "clicked",
+			   GTK_SIGNAL_FUNC(online_switch_clicked), mainwin);
+	gtk_box_pack_start(GTK_BOX(hbox_stat), online_switch, FALSE, FALSE, 0);
+
+	online_tip = gtk_tooltips_new();
+
 	ac_button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(ac_button), GTK_RELIEF_NONE);
 	GTK_WIDGET_UNSET_FLAGS(ac_button, GTK_CAN_FOCUS);
@@ -821,20 +876,25 @@ MainWindow *main_window_create(SeparateType type)
 	summaryview->messageview = messageview;
 	summaryview->window      = window;
 
+	messageview->statusbar   = statusbar;
 	messageview->mainwin     = mainwin;
 
-	mainwin->window       = window;
-	mainwin->vbox         = vbox;
-	mainwin->menubar      = menubar;
-	mainwin->menu_factory = ifactory;
-	mainwin->handlebox    = handlebox;
-	mainwin->vbox_body    = vbox_body;
-	mainwin->hbox_stat    = hbox_stat;
-	mainwin->statusbar    = statusbar;
-	mainwin->progressbar  = progressbar;
-	mainwin->statuslabel  = statuslabel;
-	mainwin->ac_button    = ac_button;
-	mainwin->ac_label     = ac_label;
+	mainwin->window         = window;
+	mainwin->vbox           = vbox;
+	mainwin->menubar        = menubar;
+	mainwin->menu_factory   = ifactory;
+	mainwin->handlebox      = handlebox;
+	mainwin->vbox_body      = vbox_body;
+	mainwin->hbox_stat      = hbox_stat;
+	mainwin->statusbar      = statusbar;
+	mainwin->progressbar    = progressbar;
+	mainwin->statuslabel    = statuslabel;
+	mainwin->online_switch  = online_switch;
+	mainwin->online_pixmap  = online_pixmap;
+	mainwin->offline_pixmap = offline_pixmap;
+	mainwin->online_tip     = online_tip;
+	mainwin->ac_button      = ac_button;
+	mainwin->ac_label       = ac_label;
 
 	/* set context IDs for status bar */
 	mainwin->mainwin_cid = gtk_statusbar_get_context_id
@@ -843,6 +903,10 @@ MainWindow *main_window_create(SeparateType type)
 		(GTK_STATUSBAR(statusbar), "Folder View");
 	mainwin->summaryview_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR(statusbar), "Summary View");
+	mainwin->messageview_cid = gtk_statusbar_get_context_id
+		(GTK_STATUSBAR(statusbar), "Message View");
+
+	messageview->statusbar_cid = mainwin->messageview_cid;
 
 	/* allocate colors for summary view and folder view */
 	summaryview->color_marked.red = summaryview->color_marked.green = 0;
@@ -854,6 +918,9 @@ MainWindow *main_window_create(SeparateType type)
 	folderview->color_new.red = (guint16)55000;
 	folderview->color_new.green = folderview->color_new.blue = 15000;
 
+	folderview->color_noselect.red = folderview->color_noselect.green =
+		folderview->color_noselect.blue = COLOR_DIM;
+
 	color[0] = summaryview->color_marked;
 	color[1] = summaryview->color_dim;
 	color[2] = folderview->color_new;
@@ -915,6 +982,10 @@ MainWindow *main_window_create(SeparateType type)
 	/* create actions menu */
 	action_update_mainwin_menu(ifactory, mainwin);
 
+	/* initialize online switch */
+	prefs_common.online_mode = !prefs_common.online_mode;
+	online_switch_clicked(online_switch, mainwin);
+
 	/* show main window */
 	gtk_widget_show(mainwin->window);
 
@@ -1029,37 +1100,85 @@ void main_window_set_summary_column(void)
 	}
 }
 
+static void main_window_set_account_selector_menu(MainWindow *mainwin,
+						  GList *account_list)
+{
+	GList *cur_ac, *cur_item;
+	GtkWidget *menuitem;
+	PrefsAccount *ac_prefs;
+
+	/* destroy all previous menu item */
+	cur_item = GTK_MENU_SHELL(mainwin->ac_menu)->children;
+	while (cur_item != NULL) {
+		GList *next = cur_item->next;
+		gtk_widget_destroy(GTK_WIDGET(cur_item->data));
+		cur_item = next;
+	}
+
+	for (cur_ac = account_list; cur_ac != NULL; cur_ac = cur_ac->next) {
+		ac_prefs = (PrefsAccount *)cur_ac->data;
+
+		menuitem = gtk_menu_item_new_with_label
+			(ac_prefs->account_name
+			 ? ac_prefs->account_name : _("Untitled"));
+		gtk_widget_show(menuitem);
+		gtk_menu_append(GTK_MENU(mainwin->ac_menu), menuitem);
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+				   GTK_SIGNAL_FUNC(account_selector_menu_cb),
+				   ac_prefs);
+	}
+}
+
+static void main_window_set_account_receive_menu(MainWindow *mainwin,
+						 GList *account_list)
+{
+	GList *cur_ac, *cur_item;
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+	PrefsAccount *ac_prefs;
+
+	menu = gtk_item_factory_get_widget(mainwin->menu_factory,
+					   "/Message/Receive");
+
+	/* search for separator */
+	for (cur_item = GTK_MENU_SHELL(menu)->children; cur_item != NULL;
+	     cur_item = cur_item->next) {
+		if (GTK_BIN(cur_item->data)->child == NULL) {
+			cur_item = cur_item->next;
+			break;
+		}
+	}
+
+	/* destroy all previous menu item */
+	while (cur_item != NULL) {
+		GList *next = cur_item->next;
+		gtk_widget_destroy(GTK_WIDGET(cur_item->data));
+		cur_item = next;
+	}
+
+	for (cur_ac = account_list; cur_ac != NULL; cur_ac = cur_ac->next) {
+		ac_prefs = (PrefsAccount *)cur_ac->data;
+
+		menuitem = gtk_menu_item_new_with_label
+			(ac_prefs->account_name ? ac_prefs->account_name
+			 : _("Untitled"));
+		gtk_widget_show(menuitem);
+		gtk_menu_append(GTK_MENU(menu), menuitem);
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+				   GTK_SIGNAL_FUNC(account_receive_menu_cb),
+				   ac_prefs);
+	}
+}
+
 void main_window_set_account_menu(GList *account_list)
 {
-	GList *cur, *cur_ac, *cur_item;
-	GtkWidget *menuitem;
+	GList *cur;
 	MainWindow *mainwin;
-	PrefsAccount *ac_prefs;
 
 	for (cur = mainwin_list; cur != NULL; cur = cur->next) {
 		mainwin = (MainWindow *)cur->data;
-
-		/* destroy all previous menu item */
-		cur_item = GTK_MENU_SHELL(mainwin->ac_menu)->children;
-		while (cur_item != NULL) {
-			GList *next = cur_item->next;
-			gtk_widget_destroy(GTK_WIDGET(cur_item->data));
-			cur_item = next;
-		}
-
-		for (cur_ac = account_list; cur_ac != NULL;
-		     cur_ac = cur_ac->next) {
-			ac_prefs = (PrefsAccount *)cur_ac->data;
-
-			menuitem = gtk_menu_item_new_with_label
-				(ac_prefs->account_name
-				 ? ac_prefs->account_name : _("Untitled"));
-			gtk_widget_show(menuitem);
-			gtk_menu_append(GTK_MENU(mainwin->ac_menu), menuitem);
-			gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-					   GTK_SIGNAL_FUNC(account_menu_cb),
-					   ac_prefs);
-		}
+		main_window_set_account_selector_menu(mainwin, account_list);
+		main_window_set_account_receive_menu(mainwin, account_list);
 	}
 }
 
@@ -1086,6 +1205,11 @@ static void main_window_show_cur_account(MainWindow *mainwin)
 	g_free(ac_name);
 }
 
+MainWindow *main_window_get(void)
+{
+	return (MainWindow *)mainwin_list->data;
+}
+
 GtkWidget *main_window_get_folder_window(MainWindow *mainwin)
 {
 	switch (mainwin->type) {
@@ -1211,6 +1335,9 @@ void main_window_toggle_message_view(MainWindow *mainwin)
 			      GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	}
 
+	if (mainwin->messageview->visible == FALSE)
+		messageview_clear(mainwin->messageview);
+
 	main_window_set_menu_sensitive(mainwin);
 
 	prefs_common.msgview_visible = mainwin->messageview->visible;
@@ -1295,6 +1422,48 @@ void main_window_get_position(MainWindow *mainwin)
 	}
 }
 
+void main_window_progress_on(MainWindow *mainwin)
+{
+	gtk_progress_set_show_text(GTK_PROGRESS(mainwin->progressbar), TRUE);
+	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), "");
+}
+
+void main_window_progress_off(MainWindow *mainwin)
+{
+	gtk_progress_set_show_text(GTK_PROGRESS(mainwin->progressbar), FALSE);
+	gtk_progress_bar_update(GTK_PROGRESS_BAR(mainwin->progressbar), 0.0);
+	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), "");
+}
+
+void main_window_progress_set(MainWindow *mainwin, gint cur, gint total)
+{
+	gchar buf[32];
+
+	g_snprintf(buf, sizeof(buf), "%d / %d", cur, total);
+	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), buf);
+	gtk_progress_bar_update(GTK_PROGRESS_BAR(mainwin->progressbar),
+				(cur == 0 && total == 0) ? 0 :
+				(gfloat)cur / (gfloat)total);
+}
+
+void main_window_toggle_online(MainWindow *mainwin, gboolean online)
+{
+	if (prefs_common.online_mode != online)
+		online_switch_clicked(mainwin->online_switch, mainwin);
+}
+
+gboolean main_window_toggle_online_if_offline(MainWindow *mainwin)
+{
+	if (!prefs_common.online_mode) {
+		if (alertpanel(_("Offline"),
+			       _("You are offline. Go online?"),
+			       _("Yes"), _("No"), NULL) == G_ALERTDEFAULT)
+			main_window_toggle_online(mainwin, TRUE);
+	}
+
+	return prefs_common.online_mode;
+}
+
 void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 {
 	GList *list;
@@ -1308,6 +1477,7 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 	}
 
 	procmsg_empty_trash();
+	statusbar_pop_all();
 
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder;
@@ -1423,7 +1593,7 @@ void main_window_set_toolbar_sensitive(MainWindow *mainwin)
 	struct {
 		GtkWidget *widget;
 		SensitiveCond cond;
-	} entry[10];
+	} entry[12];
 
 #define SET_WIDGET_COND(w, c)	\
 {				\
@@ -1437,9 +1607,13 @@ void main_window_set_toolbar_sensitive(MainWindow *mainwin)
 	SET_WIDGET_COND(mainwin->compose_btn, M_HAVE_ACCOUNT);
 	SET_WIDGET_COND(mainwin->reply_btn,
 			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
+	SET_WIDGET_COND(GTK_WIDGET_PTR(mainwin->reply_combo),
+			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
 	SET_WIDGET_COND(mainwin->replyall_btn,
 			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
 	SET_WIDGET_COND(mainwin->fwd_btn, M_HAVE_ACCOUNT|M_TARGET_EXIST);
+	SET_WIDGET_COND(GTK_WIDGET_PTR(mainwin->fwd_combo),
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 #if 0
 	SET_WIDGET_COND(mainwin->prefs_btn, M_UNLOCKED);
 	SET_WIDGET_COND(mainwin->account_btn, M_UNLOCKED);
@@ -1465,25 +1639,28 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 	GtkItemFactory *ifactory = mainwin->menu_factory;
 	SensitiveCond state;
 	gboolean sensitive;
+	GtkWidget *menu;
 	GtkWidget *menuitem;
 	FolderItem *item;
 	gchar *menu_path;
 	gint i;
+	GList *cur_item;
 
 	static const struct {
 		gchar *const entry;
 		SensitiveCond cond;
 	} entry[] = {
-		{"/File/Folder"                               , M_UNLOCKED},
-		{"/File/Add mailbox..."                       , M_UNLOCKED},
-		{"/File/Import mbox file..."                  , M_UNLOCKED},
-		{"/File/Export to mbox file..."               , M_UNLOCKED},
-		{"/File/Empty trash"                          , M_UNLOCKED},
+		{"/File/Folder"                , M_UNLOCKED},
+		{"/File/Add mailbox..."        , M_UNLOCKED},
+		{"/File/Import mbox file..."   , M_UNLOCKED},
+		{"/File/Export to mbox file...", M_UNLOCKED},
+		{"/File/Empty trash"           , M_UNLOCKED},
 
-		{"/File/Save as...", M_SINGLE_TARGET_EXIST},
-		{"/File/Print..."  , M_TARGET_EXIST},
-		/* {"/File/Close"  , M_UNLOCKED}, */
-		{"/File/Exit"      , M_UNLOCKED},
+		{"/File/Save as..."  , M_SINGLE_TARGET_EXIST},
+		{"/File/Print..."    , M_TARGET_EXIST},
+		{"/File/Work offline", M_UNLOCKED},
+		/* {"/File/Close"    , M_UNLOCKED}, */
+		{"/File/Exit"        , M_UNLOCKED},
 
 		{"/Edit/Select thread"             , M_SINGLE_TARGET_EXIST},
 
@@ -1505,27 +1682,32 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/View/Show all header"           , M_SINGLE_TARGET_EXIST},
 		{"/View/Message source"            , M_SINGLE_TARGET_EXIST},
 
-		{"/Message/Get new mail"         , M_HAVE_ACCOUNT|M_UNLOCKED},
-		{"/Message/Get from all accounts", M_HAVE_ACCOUNT|M_UNLOCKED},
-		{"/Message/Cancel receiving"     , M_INC_ACTIVE},
+		{"/Message/Receive/Get from current account"
+						 , M_HAVE_ACCOUNT|M_UNLOCKED},
+		{"/Message/Receive/Get from all accounts"
+						 , M_HAVE_ACCOUNT|M_UNLOCKED},
+		{"/Message/Receive/Cancel receiving"
+						 , M_INC_ACTIVE},
+
 		{"/Message/Compose new message"  , M_HAVE_ACCOUNT},
 		{"/Message/Reply"                , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
 		{"/Message/Reply to"             , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
 		{"/Message/Forward"              , M_HAVE_ACCOUNT|M_TARGET_EXIST},
 		{"/Message/Forward as attachment", M_HAVE_ACCOUNT|M_TARGET_EXIST},
 		{"/Message/Redirect"             , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
-		{"/Message/Re-edit"              , M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
 		{"/Message/Move..."              , M_TARGET_EXIST|M_ALLOW_DELETE},
 		{"/Message/Copy..."              , M_TARGET_EXIST|M_EXEC},
 		{"/Message/Delete"               , M_TARGET_EXIST|M_ALLOW_DELETE},
 		{"/Message/Mark"                 , M_TARGET_EXIST},
+		{"/Message/Re-edit"              , M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
 
-		{"/Tools/Add sender to address book", M_SINGLE_TARGET_EXIST},
-		{"/Tools/Filter messages"           , M_MSG_EXIST|M_EXEC},
-		{"/Tools/Create filter rule"        , M_SINGLE_TARGET_EXIST|M_UNLOCKED},
-		{"/Tools/Actions"                   , M_TARGET_EXIST|M_UNLOCKED},
-		{"/Tools/Execute"                   , M_MSG_EXIST|M_EXEC},
-		{"/Tools/Delete duplicated messages", M_MSG_EXIST|M_ALLOW_DELETE},
+		{"/Tools/Add sender to address book"   , M_SINGLE_TARGET_EXIST},
+		{"/Tools/Filter all messages in folder", M_MSG_EXIST|M_EXEC},
+		{"/Tools/Filter selected messages"     , M_TARGET_EXIST|M_EXEC},
+		{"/Tools/Create filter rule"           , M_SINGLE_TARGET_EXIST|M_UNLOCKED},
+		{"/Tools/Actions"                      , M_TARGET_EXIST|M_UNLOCKED},
+		{"/Tools/Execute"                      , M_MSG_EXIST|M_EXEC},
+		{"/Tools/Delete duplicated messages"   , M_MSG_EXIST|M_ALLOW_DELETE},
 
 		{"/Configuration", M_UNLOCKED},
 
@@ -1539,6 +1721,22 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		menu_set_sensitive(ifactory, entry[i].entry, sensitive);
 	}
 
+	menu = gtk_item_factory_get_widget(ifactory, "/Message/Receive");
+
+	/* search for separator */
+	for (cur_item = GTK_MENU_SHELL(menu)->children; cur_item != NULL;
+	     cur_item = cur_item->next) {
+		if (GTK_BIN(cur_item->data)->child == NULL) {
+			cur_item = cur_item->next;
+			break;
+		}
+	}
+
+	for (; cur_item != NULL; cur_item = cur_item->next) {
+		gtk_widget_set_sensitive(GTK_WIDGET(cur_item->data),
+					 (M_UNLOCKED & state) != 0);
+	}
+
 	main_window_menu_callback_block(mainwin);
 
 #define SET_CHECK_MENU_ACTIVE(path, active) \
@@ -1812,6 +2010,21 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	debug_print("done.\n");
 }
 
+static GtkItemFactoryEntry reply_entries[] =
+{
+	{N_("/_Reply"),			NULL, reply_cb, COMPOSE_REPLY, NULL},
+	{N_("/Reply to _all"),		NULL, reply_cb, COMPOSE_REPLY_TO_ALL, NULL},
+	{N_("/Reply to _sender"),	NULL, reply_cb, COMPOSE_REPLY_TO_SENDER, NULL},
+	{N_("/Reply to mailing _list"),	NULL, reply_cb, COMPOSE_REPLY_TO_LIST, NULL}
+};
+
+static GtkItemFactoryEntry forward_entries[] =
+{
+	{N_("/_Forward"),		NULL, reply_cb, COMPOSE_FORWARD, NULL},
+	{N_("/For_ward as attachment"), NULL, reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
+	{N_("/Redirec_t"),		NULL, reply_cb, COMPOSE_REDIRECT, NULL}
+};
+
 static void main_window_toolbar_create(MainWindow *mainwin,
 				       GtkWidget *container)
 {
@@ -1819,11 +2032,13 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 	GtkWidget *icon_wid;
 	GtkWidget *get_btn;
 	GtkWidget *getall_btn;
+	GtkWidget *send_btn;
 	GtkWidget *compose_btn;
 	GtkWidget *reply_btn;
+	ComboButton *reply_combo;
 	GtkWidget *replyall_btn;
 	GtkWidget *fwd_btn;
-	GtkWidget *send_btn;
+	ComboButton *fwd_combo;
 #if 0
 	GtkWidget *prefs_btn;
 	GtkWidget *account_btn;
@@ -1832,6 +2047,8 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 	GtkWidget *delete_btn;
 	GtkWidget *exec_btn;
 
+	gint n_entries;
+
 	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
 				  GTK_TOOLBAR_BOTH);
 	gtk_container_add(GTK_CONTAINER(container), toolbar);
@@ -1839,6 +2056,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
 	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar),
 				    GTK_TOOLBAR_SPACE_LINE);
+	gtk_widget_set_usize(toolbar, 1, -1);
 
 	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_RECEIVE);
 	get_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
@@ -1876,6 +2094,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					      icon_wid,
 					      toolbar_compose_cb,
 					      mainwin);
+
 	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_REPLY);
 	reply_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					    _("Reply"),
@@ -1884,6 +2103,17 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					    icon_wid,
 					    toolbar_reply_cb,
 					    mainwin);
+
+	n_entries = sizeof(reply_entries) / sizeof(reply_entries[0]);
+	reply_combo = gtkut_combo_button_create(reply_btn,
+						reply_entries, n_entries,
+						"<Reply>", mainwin);
+	gtk_button_set_relief(GTK_BUTTON(reply_combo->arrow),
+			      GTK_TOOLBAR(toolbar)->relief);
+	gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
+				  GTK_WIDGET_PTR(reply_combo),
+				  _("Reply to the message"), "Reply");
+
 	icon_wid = stock_pixmap_widget
 		(container, STOCK_PIXMAP_MAIL_REPLY_TO_ALL);
 	replyall_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
@@ -1893,6 +2123,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					       icon_wid,
 					       toolbar_reply_to_all_cb,
 					       mainwin);
+
 	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_FORWARD);
 	fwd_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					  _("Forward"),
@@ -1902,6 +2133,16 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					  toolbar_forward_cb,
 					  mainwin);
 
+	n_entries = sizeof(forward_entries) / sizeof(forward_entries[0]);
+	fwd_combo = gtkut_combo_button_create(fwd_btn,
+					      forward_entries, n_entries,
+					      "<Forward>", mainwin);
+	gtk_button_set_relief(GTK_BUTTON(fwd_combo->arrow),
+			      GTK_TOOLBAR(toolbar)->relief);
+	gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
+				  GTK_WIDGET_PTR(fwd_combo),
+				  _("Forward the message"), "Fwd");
+
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
 	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_CLOSE);
@@ -1960,8 +2201,10 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 	mainwin->getall_btn   = getall_btn;
 	mainwin->compose_btn  = compose_btn;
 	mainwin->reply_btn    = reply_btn;
+	mainwin->reply_combo  = reply_combo;
 	mainwin->replyall_btn = replyall_btn;
 	mainwin->fwd_btn      = fwd_btn;
+	mainwin->fwd_combo    = fwd_combo;
 	mainwin->send_btn     = send_btn;
 #if 0
 	mainwin->prefs_btn    = prefs_btn;
@@ -2093,6 +2336,51 @@ static void toolbar_account_button_pressed(GtkWidget *widget,
 }
 #endif
 
+static void toolbar_child_attached(GtkWidget *widget, GtkWidget *child,
+				   gpointer data)
+{
+	gtk_widget_set_usize(child, 1, -1);
+}
+
+static void toolbar_child_detached(GtkWidget *widget, GtkWidget *child,
+				   gpointer data)
+{
+	gtk_widget_set_usize(child, -1, -1);
+}
+
+static void online_switch_clicked(GtkWidget *widget, gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+	GtkWidget *menuitem;
+
+	menuitem = gtk_item_factory_get_item(mainwin->menu_factory,
+					     "/File/Work offline");
+
+	if (prefs_common.online_mode == TRUE) {
+		prefs_common.online_mode = FALSE;
+		gtk_widget_hide(mainwin->online_pixmap);
+		gtk_widget_show(mainwin->offline_pixmap);
+		gtk_tooltips_set_tip
+			(mainwin->online_tip, mainwin->online_switch,
+			 _("You are offline. Click the icon to go online."),
+			 NULL);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+					       TRUE);
+		inc_autocheck_timer_remove();
+	} else {
+		prefs_common.online_mode = TRUE;
+		gtk_widget_hide(mainwin->offline_pixmap);
+		gtk_widget_show(mainwin->online_pixmap);
+		gtk_tooltips_set_tip
+			(mainwin->online_tip, mainwin->online_switch,
+			 _("You are online. Click the icon to go offline."),
+			 NULL);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+					       FALSE);
+		inc_autocheck_timer_set();
+	}
+}
+
 static void ac_label_button_pressed(GtkWidget *widget, GdkEventButton *event,
 				    gpointer data)
 {
@@ -2244,6 +2532,13 @@ static void print_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	summary_print(mainwin->summaryview);
 }
 
+static void toggle_offline_cb(MainWindow *mainwin, guint action,
+			      GtkWidget *widget)
+{
+	main_window_toggle_online
+		(mainwin, !GTK_CHECK_MENU_ITEM(widget)->active);
+}
+
 static void app_exit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	if (prefs_common.confirm_on_exit) {
@@ -2407,15 +2702,12 @@ static void send_queue_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 			ret = procmsg_send_queue(folder->queue,
 						 prefs_common.savemsg);
 			statusbar_pop_all();
-			if (ret >= 0) {
+			if (ret > 0)
 				folder_item_scan(folder->queue);
-				folderview_update_item(folder->queue, TRUE);
-				if (prefs_common.savemsg && folder->outbox)
-					folderview_update_item(folder->outbox,
-							       TRUE);
-			}
 		}
 	}
+
+	folderview_update_all_updated(TRUE);
 }
 
 static void compose_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
@@ -2474,11 +2766,6 @@ static void show_all_header_cb(MainWindow *mainwin, guint action,
 				     GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
-static void reedit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
-{
-	summary_reedit(mainwin->summaryview);
-}
-
 static void mark_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	summary_mark(mainwin->summaryview);
@@ -2507,6 +2794,11 @@ static void mark_all_read_cb(MainWindow *mainwin, guint action,
 	summary_mark_all_read(mainwin->summaryview);
 }
 
+static void reedit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
+{
+	summary_reedit(mainwin->summaryview);
+}
+
 static void add_address_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
@@ -2605,7 +2897,7 @@ static void delete_duplicated_cb(MainWindow *mainwin, guint action,
 
 static void filter_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
-	summary_filter(mainwin->summaryview);
+	summary_filter(mainwin->summaryview, (gboolean)action);
 }
 
 static void execute_summary_cb(MainWindow *mainwin, guint action,
@@ -2767,12 +3059,20 @@ static void new_account_cb(MainWindow *mainwin, guint action,
 	if (!compose_get_compose_list()) account_add();
 }
 
-static void account_menu_cb(GtkMenuItem	*menuitem, gpointer data)
+static void account_selector_menu_cb(GtkMenuItem *menuitem, gpointer data)
 {
 	cur_account = (PrefsAccount *)data;
 	main_window_reflect_prefs_all();
 }
 
+static void account_receive_menu_cb(GtkMenuItem *menuitem, gpointer data)
+{
+	MainWindow *mainwin = (MainWindow *)mainwin_list->data;
+	PrefsAccount *account = (PrefsAccount *)data;
+
+	inc_account_mail(mainwin, account);
+}
+
 static void manual_open_cb(MainWindow *mainwin, guint action,
 			   GtkWidget *widget)
 {
