@@ -210,13 +210,20 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 					      (folder_table, dropfolder));
 			if (val == 0) {
 				folder_item_scan(dropfolder);
+				/* force updating */
+				if (FOLDER_IS_LOCAL(dropfolder->folder))
+					dropfolder->mtime = 0;
 				g_hash_table_insert(folder_table, dropfolder,
 						    GINT_TO_POINTER(1));
 			}
-		} else
+		} else {
 			dropfolder = dest;
+			/* force updating */
+			if (FOLDER_IS_LOCAL(dropfolder->folder))
+				dropfolder->mtime = 0;
+		}
 
-		if (folder_item_add_msg(dropfolder, tmp_file, TRUE) < 0) {
+		if (folder_item_add_msg(dropfolder, tmp_file, NULL, TRUE) < 0) {
 			fclose(mbox_fp);
 			unlink(tmp_file);
 			g_free(tmp_file);
