@@ -93,6 +93,9 @@ static gboolean mh_is_msg_changed	(Folder		*folder,
 static gint    mh_close			(Folder		*folder,
 					 FolderItem	*item);
 
+static gint    mh_scan_folder_full	(Folder		*folder,
+					 FolderItem	*item,
+					 gboolean	 count_sum);
 static gint    mh_scan_folder		(Folder		*folder,
 					 FolderItem	*item);
 static gint    mh_scan_tree		(Folder		*folder);
@@ -113,6 +116,7 @@ static gint	mh_do_move_msgs			(Folder		*folder,
 						 FolderItem	*dest,
 						 GSList		*msglist);
 
+static time_t  mh_get_mtime			(FolderItem	*item);
 static GSList  *mh_get_uncached_msgs		(GHashTable	*msg_table,
 						 FolderItem	*item);
 static MsgInfo *mh_parse_msg			(const gchar	*file,
@@ -186,9 +190,7 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 {
 	GSList *mlist;
 	GHashTable *msg_table;
-	gchar *path;
-	struct stat s;
-	gboolean scan_new = TRUE;
+	time_t cur_mtime;
 #ifdef MEASURE_TIME
 	struct timeval tv_before, tv_after, tv_result;
 
@@ -197,22 +199,10 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 
 	g_return_val_if_fail(item != NULL, NULL);
 
-	path = folder_item_get_path(item);
-	if (stat(path, &s) < 0) {
-		FILE_OP_ERROR(path, "stat");
-	} else {
-		time_t mtime;
+	cur_mtime = mh_get_mtime(item);
 
-		mtime = MAX(s.st_mtime, s.st_ctime);
-		if (item->mtime == mtime) {
-			debug_print("Folder is not modified.\n");
-			scan_new = FALSE;
-		} else
-			item->mtime = mtime;
-	}
-	g_free(path);
-
-	if (use_cache && !scan_new) {
+	if (use_cache && item->mtime == cur_mtime) {
+		debug_print("Folder is not modified.\n");
 		mlist = procmsg_read_cache(item, FALSE);
 		if (!mlist)
 			mlist = mh_get_uncached_msgs(NULL, item);
@@ -230,6 +220,8 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	} else
 		mlist = mh_get_uncached_msgs(NULL, item);
 
+	item->mtime = cur_mtime;
+
 	procmsg_set_flags(mlist, item);
 
 	mlist = procmsg_sort_msg_list(mlist, item->sort_key, item->sort_type);
@@ -372,7 +364,7 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 	if ((((MsgFileInfo *)file_list->data)->flags == NULL &&
 	    file_list->next == NULL) || dest->opened)
 		fp = NULL;
-	else if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
+	else if ((fp = procmsg_open_mark_file(dest, DATA_APPEND)) == NULL)
 		g_warning("Can't open mark file.\n");
 
 	for (cur = file_list; cur != NULL; cur = cur->next) {
@@ -395,6 +387,7 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		g_free(destfile);
 		dest->last_num++;
 		dest->total++;
+		dest->updated = TRUE;
 
 		if (fileinfo->flags) {
 			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
@@ -404,8 +397,12 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 			if (MSG_IS_UNREAD(*fileinfo->flags))
 				dest->unread++;
 		} else {
+			if (dest->unmarked_num == 0)
+				dest->new = 0;
+
 			dest->new++;
 			dest->unread++;
+			dest->unmarked_num++;
 		}
 	}
 
@@ -444,7 +441,7 @@ static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 
 	if (dest->opened)
 		fp = NULL;
-	else if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
+	else if ((fp = procmsg_open_mark_file(dest, DATA_APPEND)) == NULL)
 		g_warning(_("Can't open mark file.\n"));
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
@@ -472,8 +469,10 @@ static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 		g_free(srcfile);
 		g_free(destfile);
 		src->total--;
+		src->updated = TRUE;
 		dest->last_num++;
 		dest->total++;
+		dest->updated = TRUE;
 
 		if (fp) {
 			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
@@ -488,6 +487,8 @@ static gint mh_do_move_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 			src->unread--;
 			dest->unread++;
 		}
+
+		MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
 	}
 
 	if (fp) fclose(fp);
@@ -561,7 +562,7 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 
 	if (dest->opened)
 		fp = NULL;
-	if ((fp = procmsg_open_mark_file(dest, TRUE)) == NULL)
+	if ((fp = procmsg_open_mark_file(dest, DATA_APPEND)) == NULL)
 		g_warning(_("Can't open mark file.\n"));
 
 	for (cur = msglist; cur != NULL; cur = cur->next) {
@@ -590,6 +591,7 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, GSList *msglist)
 		g_free(destfile);
 		dest->last_num++;
 		dest->total++;
+		dest->updated = TRUE;
 
 		if (fp) {
 			SET_DEST_MSG_FLAGS(fp, dest, dest->last_num,
@@ -621,14 +623,19 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 		g_free(file);
 		return -1;
 	}
+	g_free(file);
 
 	item->total--;
+	item->updated = TRUE;
 	if (MSG_IS_NEW(msginfo->flags))
 		item->new--;
 	if (MSG_IS_UNREAD(msginfo->flags))
 		item->unread--;
+	MSG_SET_TMP_FLAGS(msginfo->flags, MSG_INVALID);
+
+	if (msginfo->msgnum == item->last_num)
+		item->last_num = mh_scan_folder_full(folder, item, FALSE);
 
-	g_free(file);
 	return 0;
 }
 
@@ -643,8 +650,10 @@ static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 	g_return_val_if_fail(path != NULL, -1);
 	val = remove_all_numbered_files(path);
 	g_free(path);
-	if (val == 0)
+	if (val == 0) {
 		item->new = item->unread = item->total = 0;
+		item->updated = TRUE;
+	}
 
 	return val;
 }
@@ -667,12 +676,12 @@ static gint mh_close(Folder *folder, FolderItem *item)
 	return 0;
 }
 
-static gint mh_scan_folder(Folder *folder, FolderItem *item)
+static gint mh_scan_folder_full(Folder *folder, FolderItem *item,
+				gboolean count_sum)
 {
 	gchar *path;
 	DIR *dp;
 	struct dirent *d;
-	struct stat s;
 	gint max = 0;
 	gint num;
 	gint n_msg = 0;
@@ -699,8 +708,7 @@ static gint mh_scan_folder(Folder *folder, FolderItem *item)
 
 	while ((d = readdir(dp)) != NULL) {
 		if ((num = to_number(d->d_name)) >= 0 &&
-		    stat(d->d_name, &s) == 0 &&
-		    S_ISREG(s.st_mode)) {
+		    dirent_is_regular_file(d)) {
 			n_msg++;
 			if (max < num)
 				max = num;
@@ -711,26 +719,36 @@ static gint mh_scan_folder(Folder *folder, FolderItem *item)
 
 	if (n_msg == 0)
 		item->new = item->unread = item->total = 0;
-	else {
+	else if (count_sum) {
 		gint new, unread, total, min, max;
 
 		procmsg_get_mark_sum
 			(item, &new, &unread, &total, &min, &max, 0);
+
 		if (n_msg > total) {
-			new += n_msg - total;
+			item->unmarked_num = new = n_msg - total;
 			unread += n_msg - total;
-		}
+		} else
+			item->unmarked_num = 0;
+
 		item->new = new;
 		item->unread = unread;
 		item->total = n_msg;
 	}
 
+	item->updated = TRUE;
+
 	debug_print(_("Last number in dir %s = %d\n"), item->path, max);
 	item->last_num = max;
 
 	return 0;
 }
 
+static gint mh_scan_folder(Folder *folder, FolderItem *item)
+{
+	return mh_scan_folder_full(folder, item, TRUE);
+}
+
 static gint mh_scan_tree(Folder *folder)
 {
 	FolderItem *item;
@@ -896,12 +914,25 @@ static gint mh_remove_folder(Folder *folder, FolderItem *item)
 }
 
 
+static time_t mh_get_mtime(FolderItem *item)
+{
+	gchar *path;
+	struct stat s;
+
+	path = folder_item_get_path(item);
+	if (stat(path, &s) < 0) {
+		FILE_OP_ERROR(path, "stat");
+		return -1;
+	} else {
+		return MAX(s.st_mtime, s.st_ctime);
+	}
+}
+
 static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 {
 	gchar *path;
 	DIR *dp;
 	struct dirent *d;
-	struct stat s;
 	GSList *newlist = NULL;
 	GSList *last = NULL;
 	MsgInfo *msginfo;
@@ -928,11 +959,6 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 	if (msg_table) {
 		while ((d = readdir(dp)) != NULL) {
 			if ((num = to_number(d->d_name)) < 0) continue;
-			if (stat(d->d_name, &s) < 0) {
-				FILE_OP_ERROR(d->d_name, "stat");
-				continue;
-			}
-			if (!S_ISREG(s.st_mode)) continue;
 
 			msginfo = g_hash_table_lookup
 				(msg_table, GUINT_TO_POINTER(num));
@@ -956,11 +982,6 @@ static GSList *mh_get_uncached_msgs(GHashTable *msg_table, FolderItem *item)
 		/* discard all previous cache */
 		while ((d = readdir(dp)) != NULL) {
 			if (to_number(d->d_name) < 0) continue;
-			if (stat(d->d_name, &s) < 0) {
-				FILE_OP_ERROR(d->d_name, "stat");
-				continue;
-			}
-			if (!S_ISREG(s.st_mode)) continue;
 
 			msginfo = mh_parse_msg(d->d_name, item);
 			if (!msginfo) continue;
@@ -999,32 +1020,32 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 	MsgInfo *msginfo;
 	MsgFlags flags;
 
-	flags.perm_flags = MSG_NEW|MSG_UNREAD;
-	flags.tmp_flags = 0;
-
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(file != NULL, NULL);
 
+	flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	flags.tmp_flags = 0;
+
 	if (item->stype == F_QUEUE) {
 		MSG_SET_TMP_FLAGS(flags, MSG_QUEUED);
 	} else if (item->stype == F_DRAFT) {
 		MSG_SET_TMP_FLAGS(flags, MSG_DRAFT);
 	}
 
+	if (stat(file, &s) < 0) {
+		FILE_OP_ERROR(file, "stat");
+		return NULL;
+	}
+	if (!S_ISREG(s.st_mode))
+		return NULL;
+
 	msginfo = procheader_parse_file(file, flags, FALSE);
 	if (!msginfo) return NULL;
 
 	msginfo->msgnum = atoi(file);
 	msginfo->folder = item;
-
-	if (stat(file, &s) < 0) {
-		FILE_OP_ERROR(file, "stat");
-		msginfo->size = 0;
-		msginfo->mtime = 0;
-	} else {
-		msginfo->size = s.st_size;
-		msginfo->mtime = s.st_mtime;
-	}
+	msginfo->size = s.st_size;
+	msginfo->mtime = s.st_mtime;
 
 	return msginfo;
 }
@@ -1122,13 +1143,16 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		else
 			entry = g_strdup(d->d_name);
 
-		if (stat(entry, &s) < 0) {
-			FILE_OP_ERROR(entry, "stat");
-			g_free(entry);
-			continue;
-		}
-
-		if (S_ISDIR(s.st_mode)) {
+		if (
+#ifdef HAVE_DIRENT_D_TYPE
+			d->d_type == DT_DIR ||
+			(d->d_type == DT_UNKNOWN &&
+#endif
+			stat(entry, &s) == 0 && S_ISDIR(s.st_mode)
+#ifdef HAVE_DIRENT_D_TYPE
+			)
+#endif
+		   ) {
 			FolderItem *new_item = NULL;
 			GNode *node;
 
@@ -1197,6 +1221,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		item->new = new;
 		item->unread = unread;
 		item->total = n_msg;
+		item->updated = TRUE;
 	}
 }
 
