@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include "main.h"
 #include "mimeview.h"
 #include "textview.h"
+#include "imageview.h"
 #include "procmime.h"
 #include "summaryview.h"
 #include "menu.h"
@@ -74,7 +75,6 @@ static void mimeview_show_image_part		(MimeView	*mimeview,
 						 MimeInfo	*partinfo);
 static void mimeview_change_view_type		(MimeView	*mimeview,
 						 MimeViewType	 type);
-static void mimeview_clear			(MimeView	*mimeview);
 
 static void mimeview_selected		(GtkCTree	*ctree,
 					 GtkCTreeNode	*node,
@@ -220,6 +220,7 @@ MimeView *mimeview_create(void)
 void mimeview_init(MimeView *mimeview)
 {
 	textview_init(mimeview->textview);
+	imageview_init(mimeview->imageview);
 }
 
 /* 
@@ -276,7 +277,6 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 {
 	GtkCTree *ctree = GTK_CTREE(mimeview->ctree);
 	GtkCTreeNode *node;
-	FILE *fp;
 
 	mimeview_clear(mimeview);
 	textview_clear(mimeview->messageview->textview);
@@ -290,6 +290,8 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 
 #if USE_GPGME
 	if (prefs_common.auto_check_signatures) {
+		FILE *fp;
+
 		if ((fp = fopen(file, "rb")) == NULL) {
 			FILE_OP_ERROR(file, "fopen");
 			return;
@@ -331,8 +333,29 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	}
 }
 
+void mimeview_clear(MimeView *mimeview)
+{
+	GtkCList *clist = GTK_CLIST(mimeview->ctree);
+
+	procmime_mimeinfo_free_all(mimeview->mimeinfo);
+	mimeview->mimeinfo = NULL;
+
+	gtk_clist_clear(clist);
+	textview_clear(mimeview->textview);
+	imageview_clear(mimeview->imageview);
+
+	mimeview->opened = NULL;
+
+	g_free(mimeview->file);
+	mimeview->file = NULL;
+
+	/* gtk_notebook_set_page(GTK_NOTEBOOK(mimeview->notebook), 0); */
+}
+
 void mimeview_destroy(MimeView *mimeview)
 {
+	textview_destroy(mimeview->textview);
+	imageview_destroy(mimeview->imageview);
 	procmime_mimeinfo_free_all(mimeview->mimeinfo);
 	g_free(mimeview->file);
 	g_free(mimeview);
@@ -497,25 +520,6 @@ static void mimeview_change_view_type(MimeView *mimeview, MimeViewType type)
 	mimeview->type = type;
 }
 
-static void mimeview_clear(MimeView *mimeview)
-{
-	GtkCList *clist = GTK_CLIST(mimeview->ctree);
-
-	procmime_mimeinfo_free_all(mimeview->mimeinfo);
-	mimeview->mimeinfo = NULL;
-
-	gtk_clist_clear(clist);
-	textview_clear(mimeview->textview);
-	imageview_clear(mimeview->imageview);
-
-	mimeview->opened = NULL;
-
-	g_free(mimeview->file);
-	mimeview->file = NULL;
-
-	/* gtk_notebook_set_page(GTK_NOTEBOOK(mimeview->notebook), 0); */
-}
-
 static void mimeview_selected(GtkCTree *ctree, GtkCTreeNode *node, gint column,
 			      MimeView *mimeview)
 {
