@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@
 #include "procheader.h"
 #include "folder.h"
 #include "session.h"
-#include "statusbar.h"
 #include "codeconv.h"
 #include "utils.h"
 #include "prefs_common.h"
@@ -261,15 +260,16 @@ static NNTPSession *news_session_get(Folder *folder)
 	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
+	if (!prefs_common.online_mode)
+		return NULL;
+
 	if (!rfolder->session) {
 		rfolder->session = news_session_new_for_folder(folder);
-		statusbar_pop_all();
 		return NNTP_SESSION(rfolder->session);
 	}
 
-	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT) {
+	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT_INTERVAL) {
 		rfolder->session->last_access_time = time(NULL);
-		statusbar_pop_all();
 		return NNTP_SESSION(rfolder->session);
 	}
 
@@ -286,7 +286,6 @@ static NNTPSession *news_session_get(Folder *folder)
 
 	if (rfolder->session)
 		rfolder->session->last_access_time = time(NULL);
-	statusbar_pop_all();
 	return NNTP_SESSION(rfolder->session);
 }
 
@@ -340,8 +339,6 @@ static GSList *news_get_article_list(Folder *folder, FolderItem *item,
 
 	alist = procmsg_sort_msg_list(alist, item->sort_key, item->sort_type);
 
-	statusbar_pop_all();
-
 	return alist;
 }
 
@@ -372,9 +369,11 @@ static gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	}
 
 	ok = news_select_group(session, item->path, NULL, NULL, NULL);
-	statusbar_pop_all();
 	if (ok != NN_SUCCESS) {
-		g_warning(_("can't select group %s\n"), item->path);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		g_free(filename);
 		return NULL;
 	}
@@ -382,9 +381,12 @@ static gchar *news_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	debug_print(_("getting article %d...\n"), num);
 	ok = news_get_article(NNTP_SESSION(REMOTE_FOLDER(folder)->session),
 			      num, filename);
-	statusbar_pop_all();
-	if (ok < 0) {
+	if (ok != NN_SUCCESS) {
 		g_warning(_("can't read article %d\n"), num);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		g_free(filename);
 		return NULL;
 	}
@@ -432,7 +434,10 @@ static gint news_scan_group(Folder *folder, FolderItem *item)
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't set group: %s\n"), item->path);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		return -1;
 	}
 
@@ -513,6 +518,7 @@ GSList *news_get_group_list(Folder *folder)
 
 	if ((fp = fopen(filename, "rb")) == NULL) {
 		NNTPSession *session;
+		gint ok;
 
 		session = news_session_get(folder);
 		if (!session) {
@@ -520,12 +526,15 @@ GSList *news_get_group_list(Folder *folder)
 			return NULL;
 		}
 
-		if (nntp_list(session) != NN_SUCCESS) {
+		ok = nntp_list(session);
+		if (ok != NN_SUCCESS) {
+			if (ok == NN_SOCKET) {
+				session_destroy(SESSION(session));
+				REMOTE_FOLDER(folder)->session = NULL;
+			}
 			g_free(filename);
-			statusbar_pop_all();
 			return NULL;
 		}
-		statusbar_pop_all();
 		if (recv_write_to_file(SESSION(session)->sock, filename) < 0) {
 			log_warning(_("can't retrieve newsgroup list\n"));
 			session_destroy(SESSION(session));
@@ -573,8 +582,6 @@ GSList *news_get_group_list(Folder *folder)
 
 	list = g_slist_sort(list, (GCompareFunc)news_group_info_compare);
 
-	statusbar_pop_all();
-
 	return list;
 }
 
@@ -625,8 +632,6 @@ gint news_post(Folder *folder, const gchar *file)
 
 	fclose(fp);
 
-	statusbar_pop_all();
-
 	return ok;
 }
 
@@ -645,6 +650,10 @@ gint news_post_stream(Folder *folder, FILE *fp)
 	ok = nntp_post(session, fp);
 	if (ok != NN_SUCCESS) {
 		log_warning(_("can't post article.\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(folder)->session = NULL;
+		}
 		return -1;
 	}
 
@@ -655,20 +664,25 @@ static gint news_get_article_cmd(NNTPSession *session, const gchar *cmd,
 				 gint num, gchar *filename)
 {
 	gchar *msgid;
+	gint ok;
 
-	if (nntp_get_article(session, cmd, num, &msgid)
-	    != NN_SUCCESS)
-		return -1;
+	ok = nntp_get_article(session, cmd, num, &msgid);
+	if (ok != NN_SUCCESS)
+		return ok;
 
 	debug_print("Message-Id = %s, num = %d\n", msgid, num);
 	g_free(msgid);
 
-	if (recv_write_to_file(SESSION(session)->sock, filename) < 0) {
+	ok = recv_write_to_file(SESSION(session)->sock, filename);
+	if (ok < 0) {
 		log_warning(_("can't retrieve article %d\n"), num);
-		return -1;
+		if (ok == -2)
+			return NN_SOCKET;
+		else
+			return NN_IOERR;
 	}
 
-	return 0;
+	return NN_SUCCESS;
 }
 
 static gint news_get_article(NNTPSession *session, gint num, gchar *filename)
@@ -716,6 +730,8 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	ok = nntp_group(session, group, num, first, last);
 	if (ok == NN_SUCCESS)
 		session->group = g_strdup(group);
+	else
+		log_warning(_("can't select group: %s\n"), group);
 
 	return ok;
 }
@@ -741,7 +757,10 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't set group: %s\n"), item->path);
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
 		return NULL;
 	}
 
@@ -772,14 +791,21 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 
 	log_message(_("getting xover %d - %d in %s...\n"),
 		    begin, end, item->path);
-	if (nntp_xover(session, begin, end) != NN_SUCCESS) {
+	ok = nntp_xover(session, begin, end);
+	if (ok != NN_SUCCESS) {
 		log_warning(_("can't get xover\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
 		return NULL;
 	}
 
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xover.\n"));
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
 			return newlist;
 		}
 
@@ -804,8 +830,13 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		}
 	}
 
-	if (nntp_xhdr(session, "to", begin, end) != NN_SUCCESS) {
+	ok = nntp_xhdr(session, "to", begin, end);
+	if (ok != NN_SUCCESS) {
 		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
 		return newlist;
 	}
 
@@ -814,6 +845,8 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xhdr.\n"));
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
 			return newlist;
 		}
 
@@ -829,8 +862,13 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		llast = llast->next;
 	}
 
-	if (nntp_xhdr(session, "cc", begin, end) != NN_SUCCESS) {
+	ok = nntp_xhdr(session, "cc", begin, end);
+	if (ok != NN_SUCCESS) {
 		log_warning(_("can't get xhdr\n"));
+		if (ok == NN_SOCKET) {
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
+		}
 		return newlist;
 	}
 
@@ -839,6 +877,8 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 	for (;;) {
 		if (sock_gets(SESSION(session)->sock, buf, sizeof(buf)) < 0) {
 			log_warning(_("error occurred while getting xhdr.\n"));
+			session_destroy(SESSION(session));
+			REMOTE_FOLDER(item->folder)->session = NULL;
 			return newlist;
 		}
 
