@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2003 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -138,7 +138,7 @@ if (!(func)) \
 { \
 	g_warning(_("failed to write configuration to file\n")); \
 	if (orig_fp) fclose(orig_fp); \
-	prefs_write_close_revert(pfile); \
+	prefs_file_close_revert(pfile); \
 	g_free(rcpath); \
 	g_free(block_label); \
 	return; \
@@ -163,7 +163,7 @@ void prefs_save_config(PrefParam *param, const gchar *label,
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 	}
 
-	if ((pfile = prefs_write_open(rcpath)) == NULL) {
+	if ((pfile = prefs_file_open(rcpath)) == NULL) {
 		g_warning(_("failed to write configuration to file\n"));
 		if (orig_fp) fclose(orig_fp);
 		g_free(rcpath);
@@ -192,7 +192,7 @@ void prefs_save_config(PrefParam *param, const gchar *label,
 	block_label = NULL;
 
 	/* write all param data to file */
-	TRY(prefs_write_param(param, pfile->fp) == 0);
+	TRY(prefs_file_write_param(pfile, param) == 0);
 
 	if (block_matched) {
 		while (fgets(buf, sizeof(buf), orig_fp) != NULL) {
@@ -208,14 +208,14 @@ void prefs_save_config(PrefParam *param, const gchar *label,
 	}
 
 	if (orig_fp) fclose(orig_fp);
-	if (prefs_write_close(pfile) < 0)
+	if (prefs_file_close(pfile) < 0)
 		g_warning(_("failed to write configuration to file\n"));
 	g_free(rcpath);
 
 	debug_print(_("Configuration is saved.\n"));
 }
 
-gint prefs_write_param(PrefParam *param, FILE *fp)
+gint prefs_file_write_param(PrefFile *pfile, PrefParam *param)
 {
 	gint i;
 	gchar buf[PREFSBUFSIZE];
@@ -248,7 +248,7 @@ gint prefs_write_param(PrefParam *param, FILE *fp)
 		}
 
 		if (buf[0] != '\0') {
-			if (fputs(buf, fp) == EOF) {
+			if (fputs(buf, pfile->fp) == EOF) {
 				perror("fputs");
 				return -1;
 			}
@@ -258,7 +258,7 @@ gint prefs_write_param(PrefParam *param, FILE *fp)
 	return 0;
 }
 
-PrefFile *prefs_write_open(const gchar *path)
+PrefFile *prefs_file_open(const gchar *path)
 {
 	PrefFile *pfile;
 	gchar *tmppath;
@@ -285,7 +285,7 @@ PrefFile *prefs_write_open(const gchar *path)
 	return pfile;
 }
 
-gint prefs_write_close(PrefFile *pfile)
+gint prefs_file_close(PrefFile *pfile)
 {
 	FILE *fp;
 	gchar *path;
@@ -334,7 +334,7 @@ gint prefs_write_close(PrefFile *pfile)
 	return 0;
 }
 
-gint prefs_write_close_revert(PrefFile *pfile)
+gint prefs_file_close_revert(PrefFile *pfile)
 {
 	gchar *tmppath;
 
