@@ -984,7 +984,8 @@ static GList *procmime_get_mime_type_list(const gchar *file)
 	GList *list = NULL;
 	FILE *fp;
 	gchar buf[BUFFSIZE];
-	gchar *p, *delim;
+	guchar *p;
+	gchar *delim;
 	MimeType *mime_type;
 
 	if ((fp = fopen(file, "rb")) == NULL) return NULL;
