@@ -122,7 +122,6 @@ struct _MimeInfo
 /* MimeInfo handling */
 
 MimeInfo *procmime_mimeinfo_new		(void);
-void procmime_mimeinfo_free		(MimeInfo	*mimeinfo);
 void procmime_mimeinfo_free_all		(MimeInfo	*mimeinfo);
 
 MimeInfo *procmime_mimeinfo_insert	(MimeInfo	*parent,
@@ -170,6 +169,7 @@ ContentType procmime_scan_mime_type	(const gchar	*mime_type);
 gchar *procmime_get_mime_type		(const gchar	*filename);
 
 EncodingType procmime_get_encoding_for_charset	(const gchar	*charset);
+EncodingType procmime_get_encoding_for_file	(const gchar	*file);
 const gchar *procmime_get_encoding_str		(EncodingType	 encoding);
 
 #endif /* __PROCMIME_H__ */
