@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "utils.h"
 #include "procmsg.h"
 #include "procheader.h"
+#include "account.h"
 #include "send_message.h"
 #include "procmime.h"
 #include "statusbar.h"
@@ -45,6 +46,16 @@ static void mark_sum_func			(gpointer	 key,
 
 static GHashTable *procmsg_read_mark_file	(FolderItem	*item);
 
+static FILE *procmsg_open_data_file		(const gchar	*file,
+						 gint		 version,
+						 DataOpenMode	 mode,
+						 gchar		*buf,
+						 size_t		 buf_size);
+static FILE *procmsg_open_cache_file_with_buffer(FolderItem	*item,
+						 DataOpenMode	 mode,
+						 gchar		*buf,
+						 size_t		 buf_size);
+
 static gint procmsg_cmp_by_mark			(gconstpointer	 a,
 						 gconstpointer	 b);
 static gint procmsg_cmp_by_unread		(gconstpointer	 a,
@@ -117,10 +128,10 @@ static gint procmsg_read_cache_data_str(FILE *fp, gchar **str)
 {
 	gchar buf[BUFFSIZE];
 	gint ret = 0;
-	size_t len;
+	guint32 len;
 
 	if (fread(&len, sizeof(len), 1, fp) == 1) {
-		if (len < 0)
+		if (len > G_MAXINT)
 			ret = -1;
 		else {
 			gchar *tmp = NULL;
@@ -150,38 +161,43 @@ static gint procmsg_read_cache_data_str(FILE *fp, gchar **str)
 		ret = -1;
 
 	if (ret < 0)
-		g_warning(_("Cache data is corrupted\n"));
+		g_warning("Cache data is corrupted\n");
 
 	return ret;
 }
 
-#define READ_CACHE_DATA(data, fp) \
-{ \
-	if (procmsg_read_cache_data_str(fp, &data) < 0) { \
-		procmsg_msginfo_free(msginfo); \
-		break; \
-	} \
+#define READ_CACHE_DATA(data, fp)				\
+{								\
+	if (procmsg_read_cache_data_str(fp, &data) < 0) {	\
+		procmsg_msginfo_free(msginfo);			\
+		procmsg_msg_list_free(mlist);			\
+		mlist = NULL;					\
+		break;						\
+	}							\
 }
 
-#define READ_CACHE_DATA_INT(n, fp) \
-{ \
-	if (fread(&n, sizeof(n), 1, fp) != 1) { \
-		g_warning(_("Cache data is corrupted\n")); \
-		procmsg_msginfo_free(msginfo); \
-		break; \
-	} \
+#define READ_CACHE_DATA_INT(n, fp)				\
+{								\
+	guint32 idata;						\
+								\
+	if (fread(&idata, sizeof(idata), 1, fp) != 1) {		\
+		g_warning("Cache data is corrupted\n");		\
+		procmsg_msginfo_free(msginfo);			\
+		procmsg_msg_list_free(mlist);			\
+		mlist = NULL;					\
+		break;						\
+	} else							\
+		n = idata;					\
 }
 
 GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 {
 	GSList *mlist = NULL;
 	GSList *last = NULL;
-	gchar *cache_file;
 	FILE *fp;
 	MsgInfo *msginfo;
 	MsgFlags default_flags;
 	gchar file_buf[BUFFSIZE];
-	gint ver;
 	guint num;
 	FolderType type;
 
@@ -214,24 +230,12 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		}
 		g_free(path);
 	}
-	cache_file = folder_item_get_cache_file(item);
-	if ((fp = fopen(cache_file, "rb")) == NULL) {
-		debug_print(_("\tNo cache file\n"));
-		g_free(cache_file);
-		return NULL;
-	}
-	setvbuf(fp, file_buf, _IOFBF, sizeof(file_buf));
-	g_free(cache_file);
 
-	debug_print(_("\tReading summary cache..."));
-
-	/* compare cache version */
-	if (fread(&ver, sizeof(ver), 1, fp) != 1 ||
-	    CACHE_VERSION != ver) {
-		debug_print(_("Cache version is different. Discarding it.\n"));
-		fclose(fp);
+	if ((fp = procmsg_open_cache_file_with_buffer
+		(item, DATA_READ, file_buf, sizeof(file_buf))) == NULL)
 		return NULL;
-	}
+
+	debug_print("Reading summary cache...");
 
 	while (fread(&num, sizeof(num), 1, fp) == 1) {
 		msginfo = g_new0(MsgInfo, 1);
@@ -272,7 +276,8 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 	}
 
 	fclose(fp);
-	debug_print(_("done.\n"));
+
+	debug_print("done.\n");
 
 	return mlist;
 }
@@ -290,7 +295,6 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	GHashTable *mark_table;
 	MsgFlags *flags;
 
-	if (!mlist) return;
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
@@ -299,6 +303,7 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	mark_table = procmsg_read_mark_file(item);
 	if (!mark_table) {
 		item->new = item->unread = item->total = g_slist_length(mlist);
+		item->updated = TRUE;
 		return;
 	}
 
@@ -347,7 +352,9 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	item->new = new;
 	item->unread = unread;
 	item->total = total;
+	item->unmarked_num = unflagged;
 	item->last_num = lastnum;
+	item->updated = TRUE;
 
 	debug_print("new: %d unread: %d unflagged: %d total: %d\n",
 		    new, unread, unflagged, total);
@@ -484,7 +491,7 @@ void procmsg_add_flags(FolderItem *item, gint num, MsgFlags flags)
 		return;
 	}
 
-	if ((fp = procmsg_open_mark_file(item, TRUE)) == NULL) {
+	if ((fp = procmsg_open_mark_file(item, DATA_APPEND)) == NULL) {
 		g_warning(_("can't open mark file\n"));
 		return;
 	}
@@ -554,7 +561,7 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 	MsgFlags *flags;
 	MsgPermFlags perm_flags;
 
-	if ((fp = procmsg_open_mark_file(item, FALSE)) == NULL)
+	if ((fp = procmsg_open_mark_file(item, DATA_READ)) == NULL)
 		return NULL;
 
 	mark_table = g_hash_table_new(NULL, g_direct_equal);
@@ -576,46 +583,96 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 	return mark_table;
 }
 
-FILE *procmsg_open_mark_file(FolderItem *item, gboolean append)
+static FILE *procmsg_open_data_file(const gchar *file, gint version,
+				    DataOpenMode mode,
+				    gchar *buf, size_t buf_size)
 {
-	gchar *markfile;
 	FILE *fp;
-	gint ver;
+	gint data_ver;
 
-	markfile = folder_item_get_mark_file(item);
+	g_return_val_if_fail(file != NULL, NULL);
 
-	if ((fp = fopen(markfile, "rb")) == NULL)
-		debug_print(_("Mark file not found.\n"));
-	else if (fread(&ver, sizeof(ver), 1, fp) != 1 || MARK_VERSION != ver) {
-		debug_print(_("Mark version is different (%d != %d). "
-			      "Discarding it.\n"), ver, MARK_VERSION);
-		fclose(fp);
-		fp = NULL;
-	}
+	if (mode == DATA_WRITE) {
+		if ((fp = fopen(file, "wb")) == NULL) {
+			FILE_OP_ERROR(file, "fopen");
+			return NULL;
+		}
+		if (change_file_mode_rw(fp, file) < 0)
+			FILE_OP_ERROR(file, "chmod");
 
-	/* read mode */
-	if (append == FALSE) {
-		g_free(markfile);
+		WRITE_CACHE_DATA_INT(version, fp);
 		return fp;
 	}
 
+	/* check version */
+	if ((fp = fopen(file, "rb")) == NULL)
+		debug_print("Mark/Cache file not found\n");
+	else {
+		if (buf && buf_size > 0)
+			setvbuf(fp, buf, _IOFBF, buf_size);
+		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1 ||
+			 version != data_ver) {
+			debug_print("Mark/Cache version is different (%d != %d). "
+				    "Discarding it.\n", data_ver, version);
+			fclose(fp);
+			fp = NULL;
+		}
+	}
+
+	if (mode == DATA_READ)
+		return fp;
+
 	if (fp) {
 		/* reopen with append mode */
 		fclose(fp);
-		if ((fp = fopen(markfile, "ab")) == NULL)
-			g_warning(_("Can't open mark file with append mode.\n"));
+		if ((fp = fopen(file, "ab")) == NULL)
+			FILE_OP_ERROR(file, "fopen");
 	} else {
 		/* open with overwrite mode if mark file doesn't exist or
 		   version is different */
-		if ((fp = fopen(markfile, "wb")) == NULL)
-			g_warning(_("Can't open mark file with write mode.\n"));
-		else {
-			ver = MARK_VERSION;
-			WRITE_CACHE_DATA_INT(ver, fp);
-		}
+		fp = procmsg_open_data_file(file, version, DATA_WRITE, buf,
+					    buf_size);
 	}
 
+	return fp;
+}
+
+static FILE *procmsg_open_cache_file_with_buffer(FolderItem *item,
+						 DataOpenMode mode,
+						 gchar *buf, size_t buf_size)
+{
+	gchar *cachefile;
+	FILE *fp;
+
+	cachefile = folder_item_get_cache_file(item);
+	fp = procmsg_open_data_file(cachefile, CACHE_VERSION, mode, buf,
+				    buf_size);
+	g_free(cachefile);
+
+	return fp;
+}
+
+FILE *procmsg_open_cache_file(FolderItem *item, DataOpenMode mode)
+{
+	gchar *cachefile;
+	FILE *fp;
+
+	cachefile = folder_item_get_cache_file(item);
+	fp = procmsg_open_data_file(cachefile, CACHE_VERSION, mode, NULL, 0);
+	g_free(cachefile);
+
+	return fp;
+}
+
+FILE *procmsg_open_mark_file(FolderItem *item, DataOpenMode mode)
+{
+	gchar *markfile;
+	FILE *fp;
+
+	markfile = folder_item_get_mark_file(item);
+	fp = procmsg_open_data_file(markfile, MARK_VERSION, mode, NULL, 0);
 	g_free(markfile);
+
 	return fp;
 }
 
@@ -670,14 +727,15 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 	return root;
 }
 
-void procmsg_move_messages(GSList *mlist)
+gint procmsg_move_messages(GSList *mlist)
 {
 	GSList *cur, *movelist = NULL;
 	MsgInfo *msginfo;
 	FolderItem *dest = NULL;
 	GHashTable *hash;
+	gint val = 0;
 
-	if (!mlist) return;
+	if (!mlist) return 0;
 
 	hash = procmsg_to_folder_hash_table_create(mlist);
 	folder_item_scan_foreach(hash);
@@ -691,28 +749,33 @@ void procmsg_move_messages(GSList *mlist)
 		} else if (dest == msginfo->to_folder) {
 			movelist = g_slist_append(movelist, msginfo);
 		} else {
-			folder_item_move_msgs(dest, movelist);
+			val = folder_item_move_msgs(dest, movelist);
 			g_slist_free(movelist);
 			movelist = NULL;
+			if (val == -1)
+				return val;
 			dest = msginfo->to_folder;
 			movelist = g_slist_append(movelist, msginfo);
 		}
 	}
 
 	if (movelist) {
-		folder_item_move_msgs(dest, movelist);
+		val = folder_item_move_msgs(dest, movelist);
 		g_slist_free(movelist);
 	}
+
+	return val == -1 ? -1 : 0;
 }
 
-void procmsg_copy_messages(GSList *mlist)
+gint procmsg_copy_messages(GSList *mlist)
 {
 	GSList *cur, *copylist = NULL;
 	MsgInfo *msginfo;
 	FolderItem *dest = NULL;
 	GHashTable *hash;
+	gint val = 0;
 
-	if (!mlist) return;
+	if (!mlist) return 0;
 
 	hash = procmsg_to_folder_hash_table_create(mlist);
 	folder_item_scan_foreach(hash);
@@ -726,18 +789,22 @@ void procmsg_copy_messages(GSList *mlist)
 		} else if (dest == msginfo->to_folder) {
 			copylist = g_slist_append(copylist, msginfo);
 		} else {
-			folder_item_copy_msgs(dest, copylist);
+			val = folder_item_copy_msgs(dest, copylist);
 			g_slist_free(copylist);
 			copylist = NULL;
+			if (val == -1)
+				return val;
 			dest = msginfo->to_folder;
 			copylist = g_slist_append(copylist, msginfo);
 		}
 	}
 
 	if (copylist) {
-		folder_item_copy_msgs(dest, copylist);
+		val = folder_item_copy_msgs(dest, copylist);
 		g_slist_free(copylist);
 	}
+
+	return val == -1 ? -1 : 0;
 }
 
 gchar *procmsg_get_message_file_path(MsgInfo *msginfo)
@@ -766,7 +833,7 @@ gchar *procmsg_get_message_file(MsgInfo *msginfo)
 
 	filename = folder_item_fetch_msg(msginfo->folder, msginfo->msgnum);
 	if (!filename)
-		g_warning(_("can't fetch message %d\n"), msginfo->msgnum);
+		debug_print(_("can't fetch message %d\n"), msginfo->msgnum);
 
 	return filename;
 }
@@ -826,7 +893,8 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 	if (!is_file_exist(file)) {
 		g_free(file);
 		file = procmsg_get_message_file(msginfo);
-		g_return_val_if_fail(file != NULL, NULL);
+		if (!file)
+			return NULL;
 	}
 
 	if ((fp = fopen(file, "rb")) == NULL) {
@@ -966,7 +1034,7 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 		} else if (hentry[H_X_MAILING_LIST].body != NULL) {
 			SET_FILTER_KEY("X-Mailing-list", H_X_MAILING_LIST);
 		} else if (hentry[H_X_SEQUENCE].body != NULL) {
-			gchar *p;
+			guchar *p;
 
 			SET_FILTER_KEY("X-Sequence", H_X_SEQUENCE);
 			p = *key;
@@ -1035,9 +1103,6 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 		queue = folder_get_default_queue();
 	g_return_val_if_fail(queue != NULL, -1);
 
-	folder_item_scan(queue);
-	if (queue->last_num < 0) return -1;
-
 	mlist = folder_item_get_msg_list(queue, FALSE);
 	mlist = procmsg_sort_msg_list(mlist, SORT_BY_NUMBER, SORT_ASCENDING);
 
@@ -1047,16 +1112,24 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 
 		file = procmsg_get_message_file(msginfo);
 		if (file) {
-			if (send_message_queue(file) < 0) {
-				g_warning(_("Sending queued message %d failed.\n"), msginfo->msgnum);
+			QueueInfo *qinfo;
+
+			qinfo = send_get_queue_info(file);
+			if (send_message_queue(qinfo) < 0) {
+				g_warning(_("Sending queued message %d failed.\n"),
+					  msginfo->msgnum);
 			} else {
 				ret++;
-				if (save_msgs)
-					procmsg_save_to_outbox
-						(queue->folder->outbox,
-						 file, TRUE);
+				if (save_msgs) {
+					FolderItem *outbox;
+					outbox = account_get_special_folder
+						(qinfo->ac, F_OUTBOX);
+					procmsg_save_to_outbox(outbox, file,
+							       TRUE);
+				}
 				folder_item_remove_msg(queue, msginfo);
 			}
+			send_queue_info_free(qinfo);
 			g_free(file);
 		}
 
@@ -1065,6 +1138,8 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 
 	g_slist_free(mlist);
 
+	queue->mtime = 0;
+
 	return ret;
 }
 
