@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,13 @@ typedef struct _MsgFileInfo	MsgFileInfo;
 #include "procmime.h"
 #include "prefs_filter.h"
 
+typedef enum
+{
+	DATA_READ,
+	DATA_WRITE,
+	DATA_APPEND
+} DataOpenMode;
+
 #define MSG_NEW			(1U << 0)
 #define MSG_UNREAD		(1U << 1)
 #define MSG_MARKED		(1U << 2)
@@ -83,6 +90,7 @@ typedef guint32 MsgPermFlags;
 #define MSG_NEWS		(1U << 20)
 #define MSG_SIGNED		(1U << 21)
 #define MSG_MIME		(1U << 29)
+#define MSG_INVALID		(1U << 30)
 #define MSG_CACHED		(1U << 31)
 
 typedef guint32 MsgTmpFlags;
@@ -121,14 +129,20 @@ typedef guint32 MsgTmpFlags;
 #define MSG_IS_IMAP(msg)		(((msg).tmp_flags & MSG_IMAP) != 0)
 #define MSG_IS_NEWS(msg)		(((msg).tmp_flags & MSG_NEWS) != 0)
 #define MSG_IS_MIME(msg)		(((msg).tmp_flags & MSG_MIME) != 0)
+#define MSG_IS_INVALID(msg)		(((msg).tmp_flags & MSG_INVALID) != 0)
 #define MSG_IS_CACHED(msg)		(((msg).tmp_flags & MSG_CACHED) != 0)
 
-#define WRITE_CACHE_DATA_INT(n, fp) \
-	fwrite(&n, sizeof(n), 1, fp)
+#define WRITE_CACHE_DATA_INT(n, fp)		\
+{						\
+	guint32 idata;				\
+						\
+	idata = (guint32)n;			\
+	fwrite(&idata, sizeof(idata), 1, fp);	\
+}
 
 #define WRITE_CACHE_DATA(data, fp)			\
 {							\
-	gint len;					\
+	size_t len;					\
 							\
 	if (data == NULL) {				\
 		len = 0;				\
@@ -213,13 +227,15 @@ void	procmsg_get_mark_sum		(FolderItem	*item,
 					 gint		*min,
 					 gint		*max,
 					 gint		 first);
+FILE   *procmsg_open_cache_file		(FolderItem	*item,
+					 DataOpenMode	 mode);
 FILE   *procmsg_open_mark_file		(FolderItem	*item,
-					 gboolean	 append);
+					 DataOpenMode	 mode);
 
 GNode  *procmsg_get_thread_tree		(GSList		*mlist);
 
-void	procmsg_move_messages		(GSList		*mlist);
-void	procmsg_copy_messages		(GSList		*mlist);
+gint	procmsg_move_messages		(GSList		*mlist);
+gint	procmsg_copy_messages		(GSList		*mlist);
 
 gchar  *procmsg_get_message_file_path	(MsgInfo	*msginfo);
 gchar  *procmsg_get_message_file	(MsgInfo	*msginfo);
