@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,12 +41,15 @@
 #include "prefs_common.h"
 #include "codeconv.h"
 #include "gtkstext.h"
+#include "statusbar.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "procmime.h"
+#include "account.h"
 #include "html.h"
 #include "compose.h"
 #include "displayheader.h"
+#include "alertpanel.h"
 
 typedef struct _RemoteURI	RemoteURI;
 
@@ -118,6 +121,18 @@ static gint text_mb_font_orig_ascent;
 static gint text_mb_font_orig_descent;
 static GdkFont *spacingfont;
 
+#define STATUSBAR_PUSH(textview, str)					    \
+{									    \
+	gtk_statusbar_push(GTK_STATUSBAR(textview->messageview->statusbar), \
+			   textview->messageview->statusbar_cid, str);	    \
+}
+
+#define STATUSBAR_POP(textview)						   \
+{									   \
+	gtk_statusbar_pop(GTK_STATUSBAR(textview->messageview->statusbar), \
+			  textview->messageview->statusbar_cid);	   \
+}
+
 static void textview_add_part		(TextView	*textview,
 					 MimeInfo	*mimeinfo,
 					 FILE		*fp);
@@ -131,6 +146,7 @@ static void textview_write_body		(TextView	*textview,
 static void textview_show_html		(TextView	*textview,
 					 FILE		*fp,
 					 CodeConverter	*conv);
+
 static void textview_write_line		(TextView	*textview,
 					 const gchar	*str,
 					 CodeConverter	*conv);
@@ -138,6 +154,7 @@ static void textview_write_link		(TextView	*textview,
 					 const gchar	*str,
 					 const gchar	*uri,
 					 CodeConverter	*conv);
+
 static GPtrArray *textview_scan_header	(TextView	*textview,
 					 FILE		*fp);
 static void textview_show_header	(TextView	*textview,
@@ -150,8 +167,6 @@ static gint textview_button_pressed	(GtkWidget	*widget,
 					 GdkEventButton	*event,
 					 TextView	*textview);
 
-static void textview_uri_list_remove_all(GSList		*uri_list);
-
 static void textview_smooth_scroll_do		(TextView	*textview,
 						 gfloat		 old_value,
 						 gfloat		 last_value,
@@ -161,6 +176,10 @@ static void textview_smooth_scroll_one_line	(TextView	*textview,
 static gboolean textview_smooth_scroll_page	(TextView	*textview,
 						 gboolean	 up);
 
+static gboolean textview_uri_security_check	(TextView	*textview,
+						 RemoteURI	*uri);
+static void textview_uri_list_remove_all	(GSList		*uri_list);
+
 
 TextView *textview_create(void)
 {
@@ -544,6 +563,21 @@ static void textview_add_parts(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 #define TEXT_INSERT(str) \
 	gtk_stext_insert(text, textview->msgfont, NULL, NULL, str, -1)
 
+void textview_show_error(TextView *textview)
+{
+	GtkSText *text;
+
+	textview_set_font(textview, NULL);
+	text = GTK_STEXT(textview->text);
+	textview_clear(textview);
+
+	gtk_stext_freeze(text);
+
+	TEXT_INSERT(_("This message can't be displayed.\n"));
+
+	gtk_stext_thaw(text);
+}
+
 void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 {
 	GtkSText *text;
@@ -654,7 +688,9 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 	/* find end point of URI */
 	for (ep_ = scanpos; *ep_ != '\0'; ep_++) {
-		if (!isgraph(*ep_) || !isascii(*ep_) || strchr("()<>\"", *ep_))
+		if (!isgraph(*(const guchar *)ep_) ||
+		    !isascii(*(const guchar *)ep_) ||
+		    strchr("()<>\"", *ep_))
 			break;
 	}
 
@@ -666,7 +702,9 @@ static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
 
 #define IS_REAL_PUNCT(ch)	(ispunct(ch) && ((ch) != '/')) 
 
-	for (; ep_ - 1 > scanpos + 1 && IS_REAL_PUNCT(*(ep_ - 1)); ep_--)
+	for (; ep_ - 1 > scanpos + 1 &&
+	       IS_REAL_PUNCT(*(const guchar *)(ep_ - 1));
+	     ep_--)
 		;
 
 #undef IS_REAL_PUNCT
@@ -708,21 +746,24 @@ static gboolean get_email_part(const gchar *start, const gchar *scanpos,
 	g_return_val_if_fail(ep != NULL, FALSE);
 
 	/* scan start of address */
-	for (bp_ = scanpos - 1; bp_ >= start && IS_RFC822_CHAR(*bp_); bp_--)
+	for (bp_ = scanpos - 1;
+	     bp_ >= start && IS_RFC822_CHAR(*(const guchar *)bp_); bp_--)
 		;
 
 	/* TODO: should start with an alnum? */
 	bp_++;
-	for (; bp_ < scanpos && !IS_ASCII_ALNUM(*bp_); bp_++)
+	for (; bp_ < scanpos && !IS_ASCII_ALNUM(*(const guchar *)bp_); bp_++)
 		;
 
 	if (bp_ != scanpos) {
 		/* scan end of address */
-		for (ep_ = scanpos + 1; *ep_ && IS_RFC822_CHAR(*ep_); ep_++)
+		for (ep_ = scanpos + 1;
+		     *ep_ && IS_RFC822_CHAR(*(const guchar *)ep_); ep_++)
 			;
 
 		/* TODO: really should terminate with an alnum? */
-		for (; ep_ > scanpos && !IS_ASCII_ALNUM(*ep_); --ep_)
+		for (; ep_ > scanpos && !IS_ASCII_ALNUM(*(const guchar *)ep_);
+		     --ep_)
 			;
 		ep_++;
 
@@ -736,6 +777,7 @@ static gboolean get_email_part(const gchar *start, const gchar *scanpos,
 	return result;
 }
 
+#undef IS_ASCII_ALNUM
 #undef IS_RFC822_CHAR
 
 static gchar *make_email_string(const gchar *bp, const gchar *ep)
@@ -952,7 +994,7 @@ void textview_write_link(TextView *textview, const gchar *str,
 
 	strcrchomp(buf);
 
-	for (bufp = buf; isspace(*bufp); bufp++)
+	for (bufp = buf; isspace(*(guchar *)bufp); bufp++)
 		gtk_stext_insert(text, textview->msgfont, NULL, NULL, bufp, 1);
 
 	r_uri = g_new(RemoteURI, 1);
@@ -972,6 +1014,7 @@ void textview_clear(TextView *textview)
 	gtk_stext_forward_delete(text, gtk_stext_get_length(text));
 	gtk_stext_thaw(text);
 
+	STATUSBAR_POP(textview);
 	textview_uri_list_remove_all(textview->uri_list);
 	textview->uri_list = NULL;
 
@@ -1537,32 +1580,95 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 static gint textview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 				    TextView *textview)
 {
+	RemoteURI *uri = NULL;
+	GSList *cur;
+	gchar *trimmed_uri;
+
 	textview->cur_pos = 
 		gtk_editable_get_position(GTK_EDITABLE(textview->text));
 
-	if (event &&
-	    ((event->button == 1 && event->type == GDK_2BUTTON_PRESS) ||
-	     event->button == 2)) {
-		GSList *cur;
+	if (!event)
+		return TRUE;
 
-		for (cur = textview->uri_list; cur != NULL; cur = cur->next) {
-			RemoteURI *uri = (RemoteURI *)cur->data;
+	for (cur = textview->uri_list; cur != NULL; cur = cur->next) {
+		RemoteURI *uri_ = (RemoteURI *)cur->data;
 
-			if (textview->cur_pos >= uri->start &&
-			    textview->cur_pos <= uri->end) {
-				if (!g_strncasecmp(uri->uri, "mailto:", 7))
-					compose_new(NULL, NULL, uri->uri + 7,
-						    NULL);
-				else
-					open_uri(uri->uri,
-						 prefs_common.uri_cmd);
-			}				
+		if (textview->cur_pos >= uri_->start &&
+		    textview->cur_pos <= uri_->end) {
+			uri = uri_;
+			break;
 		}
 	}
 
+	STATUSBAR_POP(textview);
+
+	if (!uri)
+		return TRUE;
+
+	trimmed_uri = trim_string(uri->uri, 60);
+	STATUSBAR_PUSH(textview, trimmed_uri);
+	g_free(trimmed_uri);
+
+	if ((event->button == 1 && event->type == GDK_2BUTTON_PRESS) ||
+	    event->button == 2) {
+		if (!g_strncasecmp(uri->uri, "mailto:", 7)) {
+			PrefsAccount *ac = NULL;
+			MsgInfo *msginfo = textview->messageview->msginfo;
+
+			if (msginfo && msginfo->folder)
+				ac = account_find_from_item(msginfo->folder);
+			if (ac && ac->protocol == A_NNTP)
+				ac = NULL;
+			compose_new(ac, msginfo->folder, uri->uri + 7, NULL);
+		} else if (textview_uri_security_check(textview, uri) == TRUE)
+			open_uri(uri->uri, prefs_common.uri_cmd);
+	}
+
 	return TRUE;
 }
 
+static gboolean textview_uri_security_check(TextView *textview, RemoteURI *uri)
+{
+	gchar *visible_str;
+	gboolean retval = TRUE;
+
+	if (is_uri_string(uri->uri) == FALSE)
+		return TRUE;
+
+	visible_str = gtk_editable_get_chars(GTK_EDITABLE(textview->text),
+					     uri->start, uri->end);
+	if (visible_str == NULL)
+		return TRUE;
+
+	if (strcmp(visible_str, uri->uri) != 0 && is_uri_string(visible_str)) {
+		gchar *uri_path;
+		gchar *visible_uri_path;
+
+		uri_path = get_uri_path(uri->uri);
+		visible_uri_path = get_uri_path(visible_str);
+		if (strcmp(uri_path, visible_uri_path) != 0)
+			retval = FALSE;
+	}
+
+	if (retval == FALSE) {
+		gchar *msg;
+		AlertValue aval;
+
+		msg = g_strdup_printf(_("The real URL (%s) is different from\n"
+					"the apparent URL (%s).\n"
+					"Open it anyway?"),
+				      uri->uri, visible_str);
+		aval = alertpanel(_("Warning"), msg, _("Yes"), _("No"), NULL);
+		g_free(msg);
+		if (aval == G_ALERTDEFAULT)
+			retval = TRUE;
+	}
+
+	g_free(visible_str);
+
+	return retval;
+}
+
 static void textview_uri_list_remove_all(GSList *uri_list)
 {
 	GSList *cur;
